# Author: Carsten Sachse 08-Jun-2011
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details
"""
Program to iteratively refine multiple 3D structures of helical specimens competitively from segment stacks
"""

from spring.csinfrastr.csfeatures import Features
from spring.csinfrastr.csreadinput import OptHandler
from spring.segment3d.refine.sr3d_main import SegmentRefine3d
from spring.segment3d.refine.sr3d_parameters import SegmentRefine3dPar


class SegMultiRefine3dPar(SegmentRefine3dPar):
    """
    Class to initiate default dictionary with input parameters including help and range values and status dictionary
    """
    def __init__(self):
        # package/program identity
        self.package = 'emspring'       
        self.progname = 'segmultirefine3d'
        self.proginfo = __doc__
        self.code_files = [self.progname, self.progname + '_mpi']

        self.segmentrefine3d_features = Features()
        self.feature_set = self.segmentrefine3d_features.setup(self)
        
        self.define_parameters_and_their_properties()
        self.define_program_states()


    def define_parameters_and_their_properties(self):
        super(SegMultiRefine3dPar, self).define_input_output_segmentrefine3d()
        super(SegMultiRefine3dPar, self).define_segmentrefine3d_parameters()
        
        self.feature_set = self.remove_key_from_feature_set(self.feature_set, 'Filter layer-lines option')
        
        
    def set_helical_symmetry_parameters(self):
        self.feature_set = self.set_helical_symmetries(self.feature_set)
        self.feature_set.relatives['Helical rise/rotation or pitch/number of units per turn choice']='Symmetrize helix'
        self.feature_set.relatives['Helical symmetries in Angstrom or degrees']='Symmetrize helix'
        self.feature_set = self.set_rotational_symmetries(self.feature_set)
        self.feature_set = self.set_polar_apolar_helix_choices(self.feature_set)


    def set_helical_symmetries(self, feature_set):
        inp8 = 'Helical symmetries in Angstrom or degrees'
        feature_set.parameters[inp8] = '(1.408, 22.03); (40, 60); (60, 80)'
        feature_set.hints[inp8] = 'List of semicolon-separated pairs of \'helical rise/pitch\' (Angstrom) and ' + \
        '\'rotation/number of units per turn\' to be imposed to 3D reconstructions. '
        feature_set.properties[inp8] = feature_set.file_properties(1, ['*'], None)
        feature_set.level[inp8] = 'beginner'
        
        return feature_set


    def set_reference_volume(self, feature_set):
        inp9 = 'Reference volumes'
        feature_set.parameters[inp9]='reference_vol???.hdf'
        feature_set.properties[inp9]=feature_set.file_properties(3,['hdf'],'getFiles')
        feature_set.hints[inp9]='Either single reference volume or list of references to be used for competitive ' + \
        '3D structure refinement. If multiple references specified header information provides helical ' + \
        'symmetry information. Accepted image file formats ({0}).'.format(', '.join(feature_set.properties[inp9].ext))
        feature_set = self.set_relatives_and_level(feature_set, inp9)
        
        return feature_set
    
    
    def set_rotational_symmetries(self, feature_set):
        inp7 = 'Rotational symmetries'
        feature_set.parameters[inp7] = '1, 1, 1'
        feature_set.hints[inp7] = 'List of comma-separated rotational symmetries. Position in list corresponds to ' + \
        'order of list of \'Helical symmetries\'.'
        feature_set.properties[inp7] = feature_set.file_properties(1, ['*'], None)
        feature_set.level[inp7]='intermediate'
        
        return feature_set
    
    
    def set_polar_apolar_helix_choices(self, feature_set):
        inp7 = 'Helix polarities'
        feature_set.parameters[inp7] = str('polar, apolar')
        feature_set.hints[inp7] = 'Choose whether helix is \'polar\' or \'apolar\'. Polar helices have different ' + \
        'ends at the top and bottom. Only the predominant direction within a helix will be used for the ' + \
        'reconstruction. In apolar helices they are related by 180 degree rotation. Thus each segment can be ' + \
        'inserted twice in the 3D reconstruction in opposite directions.'
        feature_set.properties[inp7] = feature_set.file_properties(1, ['*'], None)
        feature_set.level[inp7]='intermediate'

        return feature_set
    
    
    def remove_key_from_feature_set(self, feature_set, remove_key):
        attr = ['parameters', 'properties', 'hints', 'relatives', 'level']
        for each_attr in attr:
            if getattr(feature_set, each_attr).has_key(remove_key):
                getattr(feature_set, each_attr).__delitem__(remove_key)
        
        return feature_set


    def define_program_states(self):
        super(SegMultiRefine3dPar, self).define_program_states()


class SegMultiRefine3d(SegmentRefine3d):
    pass
    
            
def main():
    # Option handling
    parset = SegMultiRefine3dPar()
    mergeparset = OptHandler(parset)

    ######## Program
    stack = SegMultiRefine3d(mergeparset)
    stack.perform_iterative_projection_matching_and_3d_reconstruction()


if __name__ == '__main__':
    main()


#!/usr/bin/env python
"""
Test module to check scandotfit
"""
from spring.csinfrastr.csproductivity import TestingDirectory
from spring.micprgs.scandotfit import ScanDotFitPar, ScanDotFit
from spring.tests.csinfrastr.test_csreadinput import EndToEndTest
from spring.tests.micprgs.test_scansplit import ScanPreparation
from utilities import model_gauss_noise
import numpy as np
import os


class DotScanPreparation(ScanPreparation):
    def prepare_dot_micrograph(self):
        self.row_count = 4
        self.column_count = 6
        self.dot_tile_size_in_pixel = 89

        self.make_dot_tile()
        self.stack_reference_micrographs_to_scan()
        self.add_noise_to_scan()
        self.scan.write_image('test_dotmic.mrc')

    def make_dot_tile(self):
        self.reference_micrograph_array = np.zeros((self.dot_tile_size_in_pixel, self.dot_tile_size_in_pixel))
        self.reference_micrograph_array[int(self.dot_tile_size_in_pixel/2)][int(self.dot_tile_size_in_pixel/2)] += 1

    def add_noise_to_scan(self):
        self.noise = model_gauss_noise(0.001, self.scan.get_xsize(), self.scan.get_ysize())
        self.scan += self.noise


class TestScanDotFit(ScanDotFit, DotScanPreparation):
    """
    Test class that tests work of scandotfit
    """
    def setup(self):
        self.testingdir = TestingDirectory(os.path.splitext(os.path.basename(__file__))[0])
        self.testingdir.create()

        self.feature_set = ScanDotFitPar()
        self.prepare_dot_micrograph()

        self.feature_set.parameters['Micrograph']='test_dotmic.mrc'
        self.feature_set.parameters['Diagnostic plot']='test_visoutput.png'
        self.feature_set.parameters['Scanner step size in micrometer']=float(2500/self.dot_tile_size_in_pixel)
        self.feature_set.parameters['Width of integration in pixels']=45
        self.feature_set.parameters['Topleft coordinates of dot grid']=((59, 310))
        self.feature_set.parameters['Topright coordinates of dot grid']=((504, 336))
        
        self.feature_set.parameters['Bottomleft coordinates of dot grid']=((int(self.dot_tile_size_in_pixel/2),
        int(self.dot_tile_size_in_pixel/2)))

        super(TestScanDotFit, self).__init__(self.feature_set)


    def compare_distance_mean_stdev(self):
        assert round(self.horizontal_mean*2)/2 == 2.5
        assert round(self.vertical_mean*2)/2 == 2.5
        assert self.horizontal_stdev < 1
        assert self.vertical_stdev < 1


    def teardown(self):
        os.remove('test_dotmic.mrc')
        os.remove('test_visoutput.png')

        self.testingdir.remove()


class TestScanDotFitMain(TestScanDotFit):
    def do_test_case_df1(self):
        self.perform_scandotfit()

        self.compare_distance_mean_stdev()


class TestScanDotFitMore(TestScanDotFit):
    def do_end_to_end_test_lf_inputfile(self):
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)

    def do_end_to_end_test_lf_prompt(self):
        EndToEndTest().do_end_to_end_prompt(self.feature_set)

def main():
    tdf = TestScanDotFitMain()
    tdf.setup()
    tdf.do_test_case_df1()
        
if __name__ == '__main__':
    main()



# Author: Carsten Sachse 29-Aug-2011
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details

from PyQt4.QtCore import SIGNAL, Qt, pyqtSignal, QVariant, QThread
from PyQt4.QtGui import QMainWindow, QWidget, QSizePolicy, QMessageBox, QLabel, QAction, QIcon, QFileDialog, QVBoxLayout, \
    QTableWidget, QTableWidgetItem, QPushButton, QGridLayout, QToolBar, QApplication
from matplotlib import cm
from matplotlib.backends.backend_qt4 import NavigationToolbar2QT
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg
from matplotlib.figure import Figure
from matplotlib.ticker import FuncFormatter, LinearLocator, FormatStrFormatter
from mpl_toolkits.mplot3d import axes3d
from scipy.spatial import Delaunay
import matplotlib
import matplotlib.cbook as cbook
import numpy as np
import os

try:
    import visvis as vv
    from visvis.backends.backend_pyqt4 import Figure as vvFigure
except:
    pass

class NavigationToolbar2QTSpring(NavigationToolbar2QT):
    """
    Workaround inability to display svg icons with EMAN2/SPARX PyQT4 distribution.
    The only modification is in the extension of the icons files from svg -->png.
    """

    def _init_toolbar(self):
        self.basedir = os.path.join(matplotlib.rcParams[ 'datapath' ],'images')

        a = self.addAction(self._icon('home.png'), 'Home', self.home)
        a.setToolTip('Reset original view')
        a = self.addAction(self._icon('back.png'), 'Back', self.back)
        a.setToolTip('Back to previous view')
        a = self.addAction(self._icon('forward.png'), 'Forward', self.forward)
        a.setToolTip('Forward to next view')
        self.addSeparator()
        a = self.addAction(self._icon('move.png'), 'Pan', self.pan)
        a.setToolTip('Pan axes with left mouse, zoom with right')
        a = self.addAction(self._icon('zoom_to_rect.png'), 'Zoom', self.zoom)
        a.setToolTip('Zoom to rectangle')
        self.addSeparator()
        a = self.addAction(self._icon('subplots.png'), 'Subplots',
                self.configure_subplots)
        a.setToolTip('Configure subplots')

        a = self.addAction(self._icon('qt4_editor_options.png'),
                           'Customize', self.edit_parameters)
        a.setToolTip('Edit curves line and axes parameters')

        a = self.addAction(self._icon('filesave.png'), 'Save',
                self.save_figure)
        a.setToolTip('Save the figure')


        self.buttons = {}

        # Add the x,y location widget at the right side of the toolbar
        # The stretch factor is 1 which means any resizing of the toolbar
        # will resize this label instead of the buttons.
        if self.coordinates:
            self.locLabel = QLabel( '', self )
            self.locLabel.setAlignment(
                    Qt.AlignRight | Qt.AlignTop )
            self.locLabel.setSizePolicy(
                QSizePolicy(QSizePolicy.Expanding,
                                  QSizePolicy.Ignored))
            labelAction = self.addWidget(self.locLabel)
            labelAction.setVisible(True)

        # reference holder for subplots_adjust window
        self.adj_window = None


class NavigationToolbarQTAggSpring(NavigationToolbar2QTSpring):
    def _get_canvas(self, fig):
        return FigureCanvasQTAgg(fig)


class SpringToolbar(NavigationToolbarQTAggSpring):
    def __init__(self, parent_one=None, parent_two=None):
        NavigationToolbarQTAggSpring.__init__(self, parent_one, parent_two)
        
        # push the current view to define home if stack is empty
        
        self._views = cbook.Stack()
        self._positions = cbook.Stack()  # stack of subplot positions
#        self._views.clear()
#        self._positions.clear()
        
        self.update()
        if self._views.empty(): 
            self.push_current()
        self._idDrag=self.canvas.mpl_connect('motion_notify_event', self.mouse_move)
        self._xypress = None
        self.push_current()
        self._button_pressed = None

        self.basedir = os.path.dirname(__file__)
        
        a = self.addAction(self._icon('..{sep}images{sep}icons{sep}zoom-in.png'.format(sep=os.sep)), 'Zoom-in', self._on_custom_zoom_in)
        zoom_tip = '- also use scrollwheel to zoom in or out. Right mouse button to drag canvas.'
        a.setToolTip('Zoom-in ' + zoom_tip)
        a = self.addAction(self._icon('..{sep}images{sep}icons{sep}zoom-out.png'.format(sep=os.sep)), 'Zoom-out', self._on_custom_zoom_out)
        a.setToolTip('Zoom-out ' + zoom_tip)
        

    def compute_updated_limits_after_zoom(self, zoom_factor, limits):
        x1, x2 = limits
        zoomed_offset = abs(x2 - x1) / zoom_factor
        zoomed_limits = x1 + zoomed_offset, x2 - zoomed_offset
        
        return zoomed_limits


    def zoom_in_or_out_of_figure(self, zoom_factor):

        if self._xypress is None:
            self._xypress = [((self.canvas.figure.axes[0], 0))]
        
        self.push_current()
        for a, ind in self._xypress:
            zoomed_x = self.compute_updated_limits_after_zoom(zoom_factor, a.get_xlim())
            zoomed_y = self.compute_updated_limits_after_zoom(zoom_factor, a.get_ylim())
        
            a.set_xlim(min(zoomed_x), max(zoomed_x))
            a.set_ylim(min(zoomed_y), max(zoomed_y))
        
        self.push_current()
        self._update_view()

    def _on_custom_zoom_in(self):
        zoom_factor = 1.25
        self.zoom_in_or_out_of_figure(zoom_factor)

    def _on_custom_zoom_out(self):
        zoom_factor = -1.25
        self.zoom_in_or_out_of_figure(zoom_factor)
        
    def drag_pan(self, event):
        'the drag callback with right-mouse button'

        self.push_current()
        for a, ind in self._xypress:
            #safer to use the recorded button at the press than current button:
            #multiple button can get pressed during motion...
            a.drag_pan(1, event.key, event.x, event.y)
        self.dynamic_update()


class SpringDataExplore3d(QWidget):
    def __init__(self, xyz_data, zxy_labels, *args):
        QWidget.__init__(self, *args)
        
        self.fig = vvFigure(self)
        
        self.layout = QVBoxLayout(self)
        self.layout.addWidget(self.fig._widget)
        
        self.setLayout(self.layout)
        
        self.plot3d(xyz_data, zxy_labels)
        
        
    def compute_meshindex_for_vertices(self, xyz):
        # set to convex surface for meshing
        # find center of data
        center = np.average(xyz, 0)
        #center data
        vxyz = xyz - center
        
        vx, vy, z = np.hsplit(vxyz, 3)
        # find x-y plane distance to each point
        radials = np.sqrt(vx**2 + vy**2)
        # get max and adjust so that arctan ranges between +-45 deg
        maxRadial = np.max(radials)/0.7
        #get angle on sphere
        xi =  np.arctan2(radials / maxRadial, 1)
        #force z axis data to sphere
        vz = maxRadial * np.cos(xi)
        vxyz = np.hstack([vx, vy, vz])
        vxyz = np.append(vxyz, [[0.7, 0.7, -0.7],[-0.7, 0.7, -0.7],[0.7, -0.7, -0.7],[-0.7, -0.7, -0.7]], axis=0)
           
        # Send data to convex_hull program qhull
        dly = Delaunay(vxyz)
        meshIndx = dly.convex_hull
        
        # Check each triangle facet and flip if
        # vertex order puts back side out
        
        for index, (I1, I2, I3) in enumerate(meshIndx):
            a = vxyz[I1,:] - vxyz[I2,:] 
            b = vxyz[I2,:] - vxyz[I3,:] 
            c = np.cross(a, b)
            if np.dot(c, vxyz[I2,:]) > 0:
                meshIndx[index] = (I1, I3, I2)
                
        return meshIndx
    

    def set_aspect_ratio(self, x_distance, y_distance, z_distance, ax):
        z_aspect_ratio = max(1, int(np.max([x_distance, y_distance]) / float(z_distance)))
        y_aspect_ratio = max(1, int(np.max([x_distance, z_distance]) / float(y_distance)))
        
        ax.daspect = 1, y_aspect_ratio, z_aspect_ratio
        
        return ax
    

    def set_limits_and_labels(self, zxy_labels, x, y, z, ax):
        ax.SetLimits(rangeX=[min(x), max(x)], rangeY=[min(y), max(y)], rangeZ=[min(z), max(z)])
        ax.axis.zLabel = zxy_labels[0]
        ax.axis.xLabel = zxy_labels[1]
        ax.axis.yLabel = zxy_labels[2]
        
        return ax
    

    def plot3d(self, xyz, zxy_labels):
                
        meshIndex = self.compute_meshindex_for_vertices(xyz)
                
        x, y, z = np.hsplit(xyz, 3)
        
        x_distance = np.max(x) - np.min(x)
        y_distance = np.max(y) - np.min(y)
        z_distance = np.max(z) - np.min(z)
        
        xyz = np.append(xyz, [[np.max(x), np.max(y), np.min(z)],
                              [np.min(x), np.max(y), np.min(z)],
                              [np.max(x), np.min(y), np.min(z)],
                              [np.min(x), np.min(y), np.min(z)]], axis=0)

        #normalize depVal for color mapping   
        depVal = z
        dataRange = np.max(depVal) - np.min(depVal)
        depVal = (depVal- np.min(depVal)) /  dataRange    
    
        # Get axes
        ax = vv.gca()
        ax = self.set_aspect_ratio(x_distance, y_distance, z_distance, ax)
        ax = self.set_limits_and_labels(zxy_labels, x, y, z, ax)
        
        ms = vv.Mesh(ax, xyz, faces=meshIndex, normals=xyz)
        ms.SetValues(np.reshape(depVal,np.size(depVal)))
        
        ms.ambient = 0.9
        ms.diffuse = 0.4
        ms.colormap = vv.CM_JET
        ms.faceShading='smooth'
        ms.edgeColor = (0.5,0.5,0.5,1)
        ms.edgeShading = 'smooth'
        ms.faceColor = (1,1,1,1)
        ms.shininess = 50
        ms.specular = 0.35
        ms.emission = 0.45
      
        
class SpringDataExploreDraw(object):
        
    def extract_rise_rotation_from_symmetry_grid(self, symmetry_grid):
        helical_rises, temp = zip(*symmetry_grid[:,0].tolist())
        temp, helical_rotations = zip(*symmetry_grid[0,].tolist())
        
        return helical_rises, helical_rotations
        
        
    def set_adjustable_tick_values(self, subplot, rise_count, rotation_count, rise_start_end, rotation_start_end):
        rise_start, rise_end = rise_start_end
        rotation_start, rotation_end = rotation_start_end
        
        x = list(range(rise_count))
        y = list(range(rotation_count))
        
        def rotations(x, pos):
            new_tick = rotation_start + x * ((rotation_end - rotation_start) / float(rotation_count - 1))
            return '{0:.4}'.format(new_tick)

        def rises(y, pos):
            new_tick = rise_start + y * ((rise_end - rise_start) / float(rise_count - 1))
            return '{0:.4}'.format(new_tick)
        
        formatter_x = FuncFormatter(rotations)
        formatter_y = FuncFormatter(rises)
        subplot.xaxis.set_major_formatter(formatter_x)
        subplot.yaxis.set_major_formatter(formatter_y)
        
        return subplot


    def set_x_and_y_label(self, subplot, label_x_and_y):
        label_y, label_x = label_x_and_y
        subplot.set_xlabel(label_x)
        subplot.set_ylabel(label_y)
        
        return subplot
    

    def set_grid_values_and_labels_to_tick_axes(self, subplot, matrix, symmetry_grid):
        rise_count, rotation_count = matrix.shape
        rotation_start_end = (symmetry_grid[0][0][1], symmetry_grid[0][-1][1])
        
        rise_start_end = (symmetry_grid[0][0][0], symmetry_grid[-1][0][0])
        
        subplot = self.set_adjustable_tick_values(subplot, rise_count, rotation_count, rise_start_end,
        rotation_start_end)

        return subplot
        

    def add_colorbar_to_figure(self, label, array_img):
        cax = self.fig.add_axes()
        cbar = self.fig.colorbar(array_img, cax)
        cbar.set_label(label)


    def finish_drawing_and_update_views_for_toolbar_navigation(self):
        self.canvas.draw()
        self.mpl_toolbar.update()


    def draw_2dmatrix_plot(self, matrix, data_grid_pairs, labels, color_map='jet'):
        self.axes = self.fig.add_subplot(111)
#        self._xypress = [(self.axes, 0)]
        
        column_count, row_count = matrix.shape
        for each_column_index in range(column_count):
            line, = self.axes.plot(range(row_count), [each_column_index]*row_count, '.', markersize=0.01, picker=5)  # 5 points tolerance

        label_x_and_y = labels[1:]
        self.axes = self.set_grid_values_and_labels_to_tick_axes(self.axes, matrix, data_grid_pairs)
        self.axes = self.set_x_and_y_label(self.axes, label_x_and_y)
        
        array_img  = self.axes.imshow(matrix, cmap=color_map, origin='lower', interpolation='nearest')
            
        if labels[0] != '':
            self.add_colorbar_to_figure(labels[0], array_img)
        
        self.finish_drawing_and_update_views_for_toolbar_navigation()
        
        self.matrix = matrix
        self.symmetry_grid = data_grid_pairs
        
        
    def draw_1d_dimensions_stacked(self, matrix, data_grid_pairs, labels, picked_point):
        helical_rotation_cc = matrix[picked_point[1]].ravel()
        if len(helical_rotation_cc) > 1:
            self.axes1 = self.fig.add_subplot(211)
        else:
            self.axes1 = self.fig.add_subplot(111)
#        self._xypress = [(self.axes1, 0)]
        
        helical_rises, temp = zip(*data_grid_pairs[:,0].tolist())
        helical_rise_cc = matrix[:,picked_point[0]].ravel()
        
        heading = labels[0]
        
        self.axes1.set_ylabel(heading)
        self.axes1.set_xlabel(labels[1])
        line, = self.axes1.plot(helical_rises, helical_rise_cc, 'x', picker=5, label=labels[1])
        self.axes1.plot(helical_rises, helical_rise_cc)
        
        if len(helical_rotation_cc) > 1:
            self.axes2 = self.fig.add_subplot(212)
            self.axes2.set_ylabel(heading)
            self.axes2.set_xlabel(labels[2])
            temp, helical_rotations = zip(*data_grid_pairs[0,].tolist())
            line2, = self.axes2.plot(helical_rotations, helical_rotation_cc, 'x', picker=5, label=labels[2])
            self.axes2.plot(helical_rotations, helical_rotation_cc)
        
        self.finish_drawing_and_update_views_for_toolbar_navigation()
        
        
    def draw_3d_surface_plot(self, data, data_grid_pairs, labels):
        self.axes5 = self.fig.add_subplot(111, projection='3d')

        temp, helical_rotations = zip(*data_grid_pairs[0,].tolist())
        helical_rises, temp = zip(*data_grid_pairs[:,0].tolist())
        
        helical_rotations, helical_rises = np.meshgrid(helical_rotations, helical_rises)
        
        surf = self.axes5.plot_surface(helical_rises, helical_rotations, data, rstride=1, cstride=1, cmap=cm.jet,
        alpha=0.6, linewidth=0, antialiased=False)
        
        self.axes5.w_zaxis.set_major_locator(LinearLocator(10))
        self.axes5.w_zaxis.set_major_formatter(FormatStrFormatter('%.03f'))
        
        self.axes5.set_zlabel(labels[0])
        self.axes5.set_xlabel(labels[1])
        self.axes5.set_ylabel(labels[2])
#        cset = self.axes5.contour(helical_rises, helical_rotations, data, zdir='z')
#        cset = self.axes5.contour(helical_rises, helical_rotations, data, zdir='x')
#        cset = self.axes5.contour(helical_rises, helical_rotations, data, zdir='y')
#        fig.colorbar(surf)
        
        self.add_colorbar_to_figure(labels[0], surf)
        
        self.finish_drawing_and_update_views_for_toolbar_navigation()
        
        
    def on_draw(self, data_grid, data_grid_pairs, labels, dimension='2d', picked_point=((0,0)), color_map='jet'):
        self.previously_picked_point=picked_point
        if dimension == '1d':
            self.draw_1d_dimensions_stacked(data_grid, data_grid_pairs, labels, picked_point)
        elif dimension == '2d':
            self.draw_2dmatrix_plot(data_grid, data_grid_pairs, labels, color_map)
        elif dimension == '3d':
            self.draw_3d_surface_plot(data_grid, data_grid_pairs, labels)
    
        self.dimension = dimension
        
        
class SpringDataExplore(QMainWindow,  QThread, SpringDataExploreDraw):
    picked_left_point = pyqtSignal(tuple)
    picked_middle_point = pyqtSignal(tuple)
    picked_label = pyqtSignal(str)
    index_2d = pyqtSignal(tuple)
    
    def __init__(self, figure=None, parent=None):
        QMainWindow.__init__(self, parent)
        QThread.__init__(self, parent) 

        self.create_new_figure(figure)
        self.create_menu()
        self.create_main_frame()
        self.create_status_bar('Ready')


    def save_plot(self):
        file_choices = 'PNG (*.png)|*.png'
        
        path = unicode(QFileDialog.getSaveFileName(self, 
                        'Save file', '', 
                        file_choices))
        if path:
            self.canvas.print_figure(path, dpi=self.dpi)
            self.statusBar().showMessage('Saved to %s' % path, 2000)
    
    
    def set_about_text(self, msg):
        self.about_message = msg
        
        
    def on_about(self):
        msg = self.about_message
        QMessageBox.about(self, 'About the demo', msg.strip())
    
    
    def on_pick(self, event):
        if event.mouseevent.button in [1,2]:
            try:
                grid_point = event.artist
                xdata = grid_point.get_xdata()
                ydata = grid_point.get_ydata()
                ind = event.ind
                label = event.artist.get_label()
                picked_point = ((float(xdata[ind]), float(ydata[ind])))
                if event.mouseevent.button == 1:
                    if self.dimension == '1d':
                        self.picked_label.emit(label)
                        self.index_2d.emit(self.previously_picked_point)
                    self.picked_left_point.emit(picked_point)
                elif event.mouseevent.button == 2:
                    self.picked_middle_point.emit(picked_point)
            except TypeError:
                msg = 'Please zoom in further in unambiguously assign data grid point.'
                QMessageBox.warning(self, 'Impossible to assign data grid point', msg.strip())
        
    
    def zoom_in_or_out(self, event):
        if event.button == 'up' or 'down':
            if event.button == 'up':
                zoom_factor = 1.1
            elif event.button == 'down':
                zoom_factor = -1.1
                
            x, y = event.x, event.y
            self.mpl_toolbar._xypress=[]
            
            for i, a in enumerate(self.mpl_toolbar.canvas.figure.get_axes()):
                if x is not None and y is not None and a.in_axes(event) \
                        and a.get_navigate() and a.can_zoom():
                    self.mpl_toolbar._xypress.append(( a, i))

            self.mpl_toolbar.zoom_in_or_out_of_figure(zoom_factor)
            

    def press_pan(self, event):
        self.mpl_toolbar._xypress = []
        self.mpl_toolbar._button_pressed = 3
        for i, a in enumerate(self.canvas.figure.get_axes()):
            if event.x is not None and event.y is not None and a.in_axes(event) and a.get_navigate():
                a.start_pan(event.x, event.y, 3)
                self.mpl_toolbar._xypress.append((a, i))
                self.canvas.mpl_disconnect(self.mpl_toolbar._idDrag)
                self.mpl_toolbar._idDrag = self.canvas.mpl_connect('motion_notify_event', self.mpl_toolbar.drag_pan)
                

    def release_pan(self, event):
        if self.mpl_toolbar._button_pressed is None:
            return
        self.canvas.mpl_disconnect(self.mpl_toolbar._idDrag)
        self.mpl_toolbar._idDrag=self.canvas.mpl_connect('motion_notify_event', self.mpl_toolbar.mouse_move)
        for a, ind in self.mpl_toolbar._xypress:
            a.end_pan()
        if not self.mpl_toolbar._xypress: 
            return
        self.mpl_toolbar._xypress = []
        self.mpl_toolbar._button_pressed=None
        self.mpl_toolbar.push_current()
        self.mpl_toolbar.release(event)
        self.mpl_toolbar.draw()
        
    def drag_canvas_upon_button_press(self, event):
        if event.button == 3 and self.mpl_toolbar._button_pressed is None:
            self.press_pan(event)
            
            
    def release_canvas_upon_button_release(self, event):
        if event.button == 3 and self.mpl_toolbar._button_pressed is not None:
            self.release_pan(event)
    
    
    def create_new_figure(self, figure):
        self.main_frame = QWidget()
        if figure is None:
            self.fig = Figure()
        else:
            self.fig = figure
        self.canvas = FigureCanvasQTAgg(self.fig)
        self.canvas.setParent(self.main_frame)
        

    def create_main_frame(self):
        
        # Since we have only one plot, we can use add_axes 
        # instead of add_subplot, but then the subplot
        # configuration tool in the navigation toolbar wouldn't
        # work.
        #
        # Bind the 'pick' event for clicking on one of the bars
        #
        self.canvas.mpl_connect('pick_event', self.on_pick)
        self.canvas.mpl_connect('scroll_event', self.zoom_in_or_out)
        self.canvas.mpl_connect('button_press_event', self.drag_canvas_upon_button_press)
        self.canvas.mpl_connect('button_release_event', self.release_canvas_upon_button_release)
        
        # Create the navigation toolbar, tied to the canvas
        #
        self.mpl_toolbar = SpringToolbar(self.canvas, self.main_frame)
        
        vbox = QVBoxLayout()
        vbox.addWidget(self.mpl_toolbar)
        vbox.addWidget(self.canvas)
        
        self.main_frame.setLayout(vbox)
        self.setCentralWidget(self.main_frame)
    
    
    def create_status_bar(self, status_text):
        status_text = QLabel()
        self.statusBar().addWidget(status_text, 1)
        
        
    def create_menu(self):        
        self.file_menu = self.menuBar().addMenu('&File')
        
        load_file_action = self.create_action('&Save plot',
            shortcut='Ctrl+S', slot=self.save_plot, 
            tip='Save the plot')
        quit_action = self.create_action('&Quit', slot=self.close, 
            shortcut='Ctrl+Q', tip='Close the application')
        
        self.add_actions(self.file_menu, 
            (load_file_action, None, quit_action))
        
        self.help_menu = self.menuBar().addMenu('&Help')
        about_action = self.create_action('&About', 
            shortcut='F1', slot=self.on_about, 
            tip='About the demo')
        
        self.add_actions(self.help_menu, (about_action,))


    def add_actions(self, target, actions):
        for action in actions:
            if action is None:
                target.addSeparator()
            else:
                target.addAction(action)


    def create_action(  self, text, slot=None, shortcut=None, 
                        icon=None, tip=None, checkable=False, 
                        signal='triggered()'):
        action = QAction(text, self)
        if icon is not None:
            action.setIcon(QIcon(':/%s.png' % icon))
        if shortcut is not None:
            action.setShortcut(shortcut)
        if tip is not None:
            action.setToolTip(tip)
            action.setStatusTip(tip)
        if slot is not None:
            self.connect(action, SIGNAL(signal), slot)
        if checkable:
            action.setCheckable(True)
        return action


class SpringDataExploreTable(QTableWidget):
    primary_id_signal = pyqtSignal(int)
    def __init__(self, table_matrix, labels, *args):
        QTableWidget.__init__(self, *args)
        
        self.setSortingEnabled(False)
        self.setColumnCount(table_matrix.shape[0])
        self.setRowCount(table_matrix.shape[1])
        self.setHorizontalHeaderLabels(labels)
        self.resizeColumnsToContents()
        self.resizeRowsToContents()
        
        for column_index, each_column in enumerate(table_matrix):
            for row_index, each_data in enumerate(each_column):
                try:
                    each_item = QTableWidgetItem(float(each_data))
                except:
                    each_item = QTableWidgetItem(str(each_data))
                    
                each_item.setFlags( Qt.ItemIsSelectable |  Qt.ItemIsEnabled )

                try:
                    each_item.setData(0, QVariant(float(each_data)))
                except:
                    each_item.setData(0, QVariant(str(each_data)))
                self.setItem(row_index, column_index, each_item)
                
        self.connect(self.verticalHeader(), SIGNAL('sectionClicked(int)'), self.emit_row_from_table)
        
        self.sortItems(0)
        self.setSortingEnabled(True)
    
        self.selection = self.selectionModel() 
        
    def emit_row_from_table(self, row):
        indexes = self.selection.selectedIndexes()

        prim_id = self.item(indexes[0].row(), 0).text()
        self.primary_id_signal.emit(int(prim_id))
        

class SpringDataExploreTablePane(QWidget):
    def __init__(self, table_matrix, labels, *args):
        QWidget.__init__(self, *args)
        
        self.table_matrix = table_matrix
        self.labels = labels
        
        self.data_table = SpringDataExploreTable(self.table_matrix, self.labels)
        
        self.gridlayout = QGridLayout(self)

        self.exportAction = QAction(QIcon(os.path.join(os.path.dirname(__file__),
        '{pardir}{sep}images{sep}icons{sep}export_table.png'.format(pardir=os.pardir, sep=os.sep))),
         'Export table data', self)                         
        
        self.exportAction.setShortcut('Ctrl+E')
        self.connect(self.exportAction, SIGNAL('triggered()'), self.saveDataInFile)
        
        self.clipAction = QAction(QIcon(os.path.join(os.path.dirname(__file__),
        '{pardir}{sep}images{sep}icons{sep}clipboard.png'.format(pardir=os.pardir, sep=os.sep))), 
        'Copy data to clipboard', self)
        
        self.clipAction.setShortcut('Ctrl+C')
        self.connect(self.clipAction, SIGNAL('triggered()'), self.copySelectedCellsToClipboard)
        
        self.table_toolbar = QToolBar()
        self.table_toolbar.addAction(self.exportAction)
        self.table_toolbar.addAction(self.clipAction)
        
        self.gridlayout.addWidget(self.table_toolbar, 0, 0)
        self.gridlayout.addWidget(self.data_table, 1, 0)

        
    def saveDataInFile(self):
        fname = str(QFileDialog.getSaveFileName(self , 'Export table data', '.', 'Files ({0})'.\
        format(['*.csv', '*.tsv', '*.tab']))) 
        
        textTable = self.getSelectedCellsInTextFormat()
            
        if fname.endswith('csv'):
            delim_char = ','
        else:
            delim_char = '\t'
            
        if fname != '':
            import csv
    
            ofile = open(fname, 'w')
            ofile.write(delim_char.join(self.labels) + '\n')
            out = csv.writer(ofile, delimiter=delim_char)
            for each_row in textTable:
                out.writerow(each_row)
            
            ofile.close()
            
            
    def getSelectedCellsInTextFormat(self):
        indexes = self.data_table.selection.selectedIndexes()

        if indexes == []:
            swapped_table = self.table_matrix.T
            new_array = np.array([str(each_value) for each_value in swapped_table.reshape(swapped_table.size)])
            new_array = new_array.reshape(swapped_table.shape)
            textTable = new_array.tolist()
        else:
            columns = indexes[-1].column() - indexes[0].column() + 1
            rows = len(indexes) / columns
            textTable = [[''] * columns for i in list(range(rows))]
    
            for i, index in enumerate(indexes):
                textTable[i % rows][i / rows] = str(self.data_table.item(index.row(), index.column()).text())
    
        return textTable
    
        
    def copySelectedCellsToClipboard(self):
        textTable = self.getSelectedCellsInTextFormat()
        QApplication.clipboard().setText('\n'.join(('\t'.join(i) for i in textTable)))
        
class SpringCommon(object):
    def setup_spring_page_top(self, obj, feature_set):
        filename = ''
        if type(feature_set.parameters.values()[0]) is str:
            if os.path.exists(feature_set.parameters.values()[0]):
                filename = ': ' + os.path.basename(feature_set.parameters.values()[0])
                
        obj.setWindowTitle(feature_set.progname.title() + filename)
        obj.program_description = QLabel()
        obj.program_description.setText(feature_set.proginfo)
        obj.exit_button = QPushButton()
        obj.exit_button.setText('Exit')
        
        exitIcon = QIcon(os.path.join(os.path.dirname(__file__),
        '{pardir}{sep}images{sep}icons{sep}exit.png'.format(pardir=os.pardir, sep=os.sep)))
        
        obj.exit_button.setIcon(exitIcon)
        obj.exit_button.setShortcut('Ctrl+Q')
        obj.connect(obj.exit_button, SIGNAL('clicked()'), obj.close)
        obj.layout = QGridLayout()
        obj.layout.addWidget(obj.program_description, 0, 0, 1, 2)
        obj.layout.addWidget(obj.exit_button, 0, 4, 1, 1)
        
        return obj



#!/usr/bin/env python
"""
Test module to check michelixtrace module
"""
from EMAN2 import EMData
from fundamentals import window2d, rot_shift2D, shift2d
from glob import glob
from multiprocessing import cpu_count
import os
from spring.csinfrastr.csproductivity import TestingDirectory
from spring.micprgs.michelixtrace import MicHelixTrace, MicHelixTracePar
from spring.tests.csinfrastr.test_csreadinput import EndToEndTest
from spring.tests.segment2d.test_segment import MicrographHelixPreparation
from utilities import model_circle


class TestMicHelixTracePrepare(object):
    def prepare_micrograph_and_reference(self):
        MicrographHelixPreparation().prepare_ctf_micrograph(self.micrograph_test_file, self.avg_defocus, self.cs,
        self.voltage, self.pixelsize, self.ampcont, self.astigmatism, self.astigmatism_angle)

        left_x, left_y = 88, 80
        img = EMData()
        img.read_image(self.micrograph_test_file)
        shifted_img_zero = shift2d(img, img.get_xsize() / 4.0, img.get_ysize() / 4.0)
        shifted_img_one = shift2d(img, img.get_xsize() / 2, img.get_ysize() / 3)
        shifted_img_two = rot_shift2D(shifted_img_one, 103, 220, 300)
        shifted_img_three = rot_shift2D(shifted_img_one, -45, -350, 100)
        shifted_img_four = rot_shift2D(shifted_img_one, 103, 150, 300)
        shifted_img_five = rot_shift2D(shifted_img_one, 90, -40, -40)

        new_mic = img + shifted_img_zero + shifted_img_one + shifted_img_two + shifted_img_three + shifted_img_four + \
        shifted_img_five

        new_mic.write_image(self.micrograph_test_file)
        
        wimg = window2d(img, 150, 150, 'l', left_x, left_y)
        wimg = rot_shift2D(wimg, -45)
        wimg *= model_circle(75, 150, 150)
        wimg.write_image(self.ref_test_image)

    def write_groud_truth_coord_file(self):
        coord_txt = \
        "630     577     100     100     -1\n" + \
        "523     748     100     100     -2\n" + \
        "616     442     100     100     -1\n" + \
        "482     309     100     100     -2\n" + \
        "453     754     100     100     -1\n" + \
        "563     582     100     100     -2\n" + \
        "262     388     100     100     -1\n" + \
        "310     342     100     100     -2\n" + \
        "238     229     100     100     -1\n" + \
        "314     308     100     100     -2\n" + \
        "403     245     100     100     -1\n" + \
        "336     309     100     100     -2\n" + \
        "-18     -24     100     100     -1\n" + \
        "115     112     100     100     -2\n" + \
        "223     668     100     100     -1\n" + \
        "221     463     100     100     -2"

        coord_f = open(self.helix_box_file, 'w')
        coord_f.write(coord_txt)
        coord_f.close()


    def check_detected_helix_count(self, expected_count=4):
        f = open(os.path.basename(self.helix_box_file), 'r')
        helix_count = 0
        for each_line in f.readlines():
            if '-2' in each_line:
                helix_count += 1
        
        print helix_count
        assert expected_count - 1 <= helix_count 


class TestMicHelixTrace(TestMicHelixTracePrepare, MicHelixTrace):
    def setup_of_common_non_mpi_parameters(self):
        self.feature_set = MicHelixTracePar()
        self.setup_main_michelixtrace_parameters()
        
    def setup_main_michelixtrace_parameters(self):
        self.micrograph_test_file = 'test_mic.hdf'
        self.test_diagnostic = 'test_diagnostic.pdf'
        self.helix_box_file = os.path.join(os.pardir, os.path.splitext(self.micrograph_test_file)[0] + os.extsep + 'box')
        self.avg_defocus = 20000
        self.cs = 2.0
        self.voltage = 200.0
        self.pixelsize = 5.0
        self.ampcont = 0.10
        self.astigmatism = 1000
        self.astigmatism_angle = 35.0
        self.ref_test_image = 'test_reference.hdf'

        self.prepare_micrograph_and_reference()
        
        self.feature_set.parameters['Micrographs']=self.micrograph_test_file
        self.feature_set.parameters['Diagnostic plot pattern']=self.test_diagnostic
        self.feature_set.parameters['Helix reference']=self.ref_test_image
        self.feature_set.parameters['Invert option']=False
        self.feature_set.parameters['Estimated helix width in Angstrom']=300
            
        self.feature_set.parameters['Pixel size in Angstrom'] = self.pixelsize
        self.feature_set.parameters['Binning option'] = True
        self.feature_set.parameters['Binning factor'] = 4
        self.feature_set.parameters['Tile size power spectrum in Angstrom'] = 640
        self.feature_set.parameters['Tile overlap in percent']= 80

        self.feature_set.parameters['Alpha threshold cc-map']=0.0001
        self.feature_set.parameters['Absolute threshold option cc-map']=False
        self.feature_set.parameters['Absolute threshold cc-map']=0.2
        self.feature_set.parameters['Order fit']= 2
        self.feature_set.parameters['Minimum and maximum helix length']=((500, 1500))
        self.feature_set.parameters['Box file coordinate step']=70.0
        self.feature_set.parameters['Pruning cutoff bending']=2.0
        self.feature_set.parameters['Compute performance score']=False
        self.feature_set.parameters['Parameter search option']=False
        self.feature_set.parameters['Manually traced helix file']=self.helix_box_file

        self.feature_set.parameters['MPI option']=False
        self.feature_set.parameters['Number of CPUs']=min(8, cpu_count())
            
        self.feature_set.parameters['Temporary directory']=os.path.abspath(os.curdir)

    def setup(self):
        self.testingdir = TestingDirectory(os.path.splitext(os.path.basename(__file__))[0])
        self.testingdir.create()
        
        self.setup_of_common_non_mpi_parameters()
        super(TestMicHelixTrace, self).__init__(self.feature_set)


    def remove_mic_dir_including_box_files(self):
        [os.remove(each_file) for each_file in glob('test_mic_groundtruth/*'.format(sep=os.extsep))]
        test_mic_dir = os.path.splitext(self.micrograph_test_file)[0] + '_groundtruth'
        if os.path.exists(test_mic_dir):
            os.rmdir(test_mic_dir)

    def teardown(self):
        pass
        os.remove(self.micrograph_test_file)
        os.remove(self.test_diagnostic)
        if os.path.exists(self.helix_box_file):
            os.remove(self.helix_box_file)
            os.remove('spring.db')
        os.remove(self.ref_test_image)
               
        self.remove_mic_dir_including_box_files()
                    
        self.testingdir.remove()


class TestMicHelixTraceMain(TestMicHelixTrace):
    def do_test_case_mt1(self):
        """
        Mictrace test to trace filaments using a reference
        """
        self.trace_helices()
        self.check_detected_helix_count(5)


class TestMicHelixTraceMore(TestMicHelixTrace):
    def do_test_case_mt2(self):
        """
        Mictrace test empty micrograph
        """
        super(TestMicHelixTrace, self).__init__(self.feature_set)
      
        MicrographHelixPreparation().prepare_ctf_micrograph(self.micrograph_test_file, self.avg_defocus, self.cs,
        self.voltage, self.pixelsize, self.ampcont, self.astigmatism, self.astigmatism_angle, helix=False)
              
        self.trace_helices()
        assert not os.path.exists(os.path.basename(self.helix_box_file))
  
  
    def do_test_case_mt3(self):
        """
        Mictrace test parameter search including parameter search
        """
        self.feature_set.parameters['Compute performance score']=True
        self.feature_set.parameters['Parameter search option']=True
        super(TestMicHelixTrace, self).__init__(self.feature_set)
     
        self.write_groud_truth_coord_file()
        self.trace_helices()
        os.rename(self.helix_box_file, self.test_diagnostic)
        assert os.path.exists('trace_grid.db')
        os.remove('trace_grid.db')
 
 
    def do_test_case_mt4(self):
        """
        Mictrace test recall/precision computation but no parameter search
        """
        self.feature_set.parameters['Compute performance score']=True
        self.feature_set.parameters['Parameter search option']=False
        super(TestMicHelixTrace, self).__init__(self.feature_set)
     
        self.write_groud_truth_coord_file()
        self.trace_helices()
        os.remove(self.helix_box_file)


    def do_test_case_mt5(self):
        """
        Mictrace test absolute threshold option
        """
        self.feature_set.parameters['Absolute threshold option cc-map']=True
        super(TestMicHelixTrace, self).__init__(self.feature_set)
     
        self.trace_helices()
        self.check_detected_helix_count(5)


class TestMicHelixTraceEndToEnd(TestMicHelixTrace):
    def do_end_to_end_test_mt_inputfile(self):
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)

    def do_end_to_end_test_mt_prompt(self):
        EndToEndTest().do_end_to_end_prompt(self.feature_set)


class TestMicHelixTraceMpi(TestMicHelixTrace):
    def do_end_to_end_test_mt_inputfile_with_MPI(self):
        self.feature_set.parameters['MPI option'] = True
        self.feature_set.parameters['Number of CPUs']=2
        
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)


class TestMicHelixTraceMpiMore(TestMicHelixTrace):
    def do_end_to_end_test_mt_inputfile_with_MPI_param(self):
        self.feature_set.parameters['Compute performance score']=True
        self.feature_set.parameters['Parameter search option']=True
        self.feature_set.parameters['MPI option'] = True
        self.feature_set.parameters['Number of CPUs']=2
        
        self.write_groud_truth_coord_file()
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)
        os.rename(self.helix_box_file, self.test_diagnostic)
        assert os.path.exists('trace_grid.db')
        os.remove('trace_grid.db')

    def do_end_to_end_test_mt_inputfile_with_MPI_comp(self):
        self.feature_set.parameters['Compute performance score']=True
        self.feature_set.parameters['Parameter search option']=False
        self.feature_set.parameters['MPI option'] = True
        self.feature_set.parameters['Number of CPUs']=2
        
        self.write_groud_truth_coord_file()
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)
        os.rename(self.helix_box_file, self.test_diagnostic)
        assert os.path.exists('trace_grid.db')
        os.remove('trace_grid.db')

def main():
    tseg = TestMicHelixTraceMain()
    tseg.setup()
    tseg.do_test_case_mt1()
        
if __name__ == '__main__':
    main()



# Author: Carsten Sachse 08-Jun-2011
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details
from EMAN2 import EMData, EMUtil, Util, Transform
from collections import namedtuple
import gc
import os
from spring.csinfrastr.csdatabase import SpringDataBase, grid_base, GridRefineTable, GridTable
from spring.csinfrastr.csproductivity import OpenMpi, Temporary
from spring.csinfrastr.csreadinput import OptHandler
from spring.micprgs.scansplit import Micrograph
from spring.segment2d.segment import Segment
from spring.segment2d.segmentexam import SegmentExam
from spring.segment3d.segclassreconstruct_compare import SegClassReconstructCompareVisual
from spring.segment3d.segclassreconstruct_prep import SegClassReconstructPar
import sys

import numpy as np


class SegClassReconstruct3d(SegClassReconstructCompareVisual):
            
    def determine_best_in_plane_rotation_angle(self, class_avg, inplane_rotations, helix_mask=None, helix_shift_x=0.0):
        
        sigmas = []
        for each_inplane_angle in inplane_rotations:
            rot_class_avg = Segment().shift_and_rotate_image(class_avg, each_inplane_angle, helix_shift_x, 0.0)
            if helix_mask is not None:
                rot_class_avg *= helix_mask
            class_avg_profile = SegmentExam().project_helix(rot_class_avg)
            stat = Micrograph().get_statistics_from_image(class_avg_profile)
            sigmas.append(stat.sigma)
            
        rotation = inplane_rotations[np.argmax(sigmas)]
        
        return rotation
    

    def center_and_rotate_image_by_helix_projection(self, helix_width_pixel, inplane_rotations, class_avg, segment_size,
    helix_height):
        helix_mask = SegmentExam().make_smooth_rectangular_mask(helix_width_pixel * 1.5, helix_height, segment_size)
        class_avg_profile = SegmentExam().project_helix(class_avg * helix_mask)
        centerx, symimg_par = Segment().minimize_xposition(class_avg_profile, 0.3 * helix_width_pixel)
        
        inplane_rotation = self.determine_best_in_plane_rotation_angle(class_avg, inplane_rotations, helix_mask,
        centerx)
        
        class_avg_centered = Segment().shift_and_rotate_image(class_avg, inplane_rotation, centerx, 0.0)

        return class_avg_centered, centerx, inplane_rotation


    def center_class_avg(self, infile, centered_class_avg_file, helix_width_pixel, percent_reconstruction_size,
    out_of_plane_tilt_angle):
        class_avg = EMData()
        class_avg.read_image(infile)
        segment_size = class_avg.get_ysize()
        reconstruction_length = percent_reconstruction_size * segment_size / 100
        
        inplane_rotations = np.arange(-10.0, 10.0, 0.05)
        
        class_avg_centered, centerx, inplane_rotation = \
        self.center_and_rotate_image_by_helix_projection(helix_width_pixel, inplane_rotations, class_avg, segment_size,
        reconstruction_length)
        
        class_avg_centered.write_image(centered_class_avg_file)
        
        self.centerx = centerx
        self.inplane_rotation = inplane_rotation
        phi, theta, psi, sx, sy = self.get_standard_proj_parameters(out_of_plane_tilt_angle)
        
        trans = Transform({'type':'SPIDER', 'phi':phi, 'theta':theta, 'psi': psi - self.inplane_rotation, 
        'tx': sx - self.centerx, 'ty':sy, 'scale':1.0})
        
        class_avg.set_attr('xform.projection', trans)
        class_avg.write_image(os.path.basename(infile))
        
        return centered_class_avg_file


    def generate_symmetry_grid_file_name(self, symmetry_pair, volume_name, rise_rot_or_pitch_unit_choice='rise/rotation'):
        """
        >>> from spring.segment3d.segclassreconstruct import SegClassReconstruct
        >>> SegClassReconstruct().generate_symmetry_grid_file_name([12,50], 'rec_vol.hdf', 'rise/rotation')
        'rec_vol_rise12_rot50.hdf'
        >>> SegClassReconstruct().generate_symmetry_grid_file_name([12,50], 'rec_vol.hdf', 'pitch/unit_number')
        'rec_vol_pitch12_unitno50.hdf'
        >>> SegClassReconstruct().generate_symmetry_grid_file_name([12.003,50.333356], 'rec_vol.hdf', 'rise/rotation')
        'rec_vol_rise12003_rot50333356.hdf'
        >>> SegClassReconstruct().generate_symmetry_grid_file_name([12,50], '/test/rec_vol.hdf', 'pitch/unit_number')
        '/test/rec_vol_pitch12_unitno50.hdf'
        """
        
        if rise_rot_or_pitch_unit_choice == 'rise/rotation':
            symmetry_pair_no_dots = '_rise' + ''.join(str(symmetry_pair[0]).split('.')) + '_rot' + \
            ''.join(str(symmetry_pair[1]).split('.'))
        elif rise_rot_or_pitch_unit_choice == 'pitch/unit_number':
            symmetry_pair_no_dots = '_pitch' + ''.join(str(symmetry_pair[0]).split('.')) + '_unitno' + \
            ''.join(str(symmetry_pair[1]).split('.'))
        symmetry_helix_volume = '{name}{pair}{ext}'.format(name=os.path.splitext(volume_name)[0],
        pair=symmetry_pair_no_dots, ext=os.path.splitext(volume_name)[-1])
        
        return symmetry_helix_volume
    

    def determine_trimmed_image_size(self, infile, percent_reconstruction_size, helixwidth_pix, pixelsize):
        temp = EMData()
        temp.read_image(infile, 0)
        segment_size_in_pixel = temp.get_xsize()
        
        helixheight_pix = int(round(segment_size_in_pixel * percent_reconstruction_size / 100.0, -1))
        trimmed_image_size = max(helixwidth_pix, helixheight_pix)
        if trimmed_image_size - helixheight_pix == 1:
            helixheight_pix -= 1
        
        if helixheight_pix * pixelsize < 2:
            msg = 'Your resulting helix height will be smaller than 2 Angstrom ' + \
            '(\'Percent image reconstruction size\' * segment size * pixelsize) = '
            '({0} * {1} * {2}) < 2 Angstrom '.format(percent_reconstruction_size * segment_size_in_pixel * pixelsize)
            raise ValueError(msg)
            
        return segment_size_in_pixel, trimmed_image_size, helixheight_pix
    
    
    def determine_symmetry_view_count_for_average_helical_rise(self, symmetry_sequence, segment_size_in_pixel,
    trimmed_image_size):
        helical_rises = np.unique([each_pair[0] for each_pair in symmetry_sequence])
        avg_helical_rise = np.mean(helical_rises)
        
        symmetry_views_count = self.determine_symmetry_view_count(segment_size_in_pixel, trimmed_image_size,
        (avg_helical_rise, 0), self.pixelsize, self.minimum_image_count_3d)
        
        return symmetry_views_count


    def determine_symmetry_view_count(self, segment_size_in_pixel, trimmed_image_size, each_symmetry_pair, pixelsize,
    minimum_image_count_3d=8):
        """
        >>> sg = SegClassReconstruct()
        >>> sg.determine_symmetry_view_count(530, 500, ((77.3, 34)), 1, 0)
        1
        >>> sg.determine_symmetry_view_count(575, 500, ((77.3, 34)), 1, 0)
        1
        >>> sg.determine_symmetry_view_count(100, 90, ((10, 34)), 10)
        10
        >>> sg.determine_symmetry_view_count(530, 500, ((0, 0)), 1, 0)
        1
        >>> sg.determine_symmetry_view_count(100, 90, ((20, 34)), 10, 8) #doctest: +NORMALIZE_WHITESPACE
        Traceback (most recent call last):
          File '<stdin>', line 1, in <module>
          File 'spring/segment2d/segclassreconstruct.py', line 444, in determine_symmetry_view_count
            raise ValueError, error_message
        ValueError: Number of computed symmetry-related views useful for reconstruction is smaller than 8 (with 
        segment_size of 90/100 pixels and helical rise: 20). No sensible 3D reconstruction possible. Either decrease 
        'Helical rise' < 12 Angstrom or 'percent of reconstruction size' < 84 % parameters. In addition, you can also 
        decrease 'Minimum number of images for 3D reconstruction'.
        
        """
        helical_rise = each_symmetry_pair[0]
        if each_symmetry_pair != (0, 0):
            symmetry_views_count = max(1, int((segment_size_in_pixel - trimmed_image_size) / \
            (helical_rise / pixelsize) + 0.5))
        else:
            symmetry_views_count = 1
        
        if symmetry_views_count < minimum_image_count_3d:
            new_percentage = 100 * (segment_size_in_pixel - (helical_rise * 8 / pixelsize)) / segment_size_in_pixel
            if new_percentage < 0:
                new_percentage = 0
            error_message = 'Number of computed symmetry-related views useful for reconstruction is smaller than ' + \
            '{min_img} '.format(min_img=minimum_image_count_3d)
            
            error_message += '(with segment_size of {0}/{1} pixels '.format(trimmed_image_size, segment_size_in_pixel)+\
            'and helical rise: {0}). No sensible 3D reconstruction possible. '.format(helical_rise)
            
            new_rise=(segment_size_in_pixel-trimmed_image_size) * pixelsize/minimum_image_count_3d
            error_message += 'Either decrease \'Helical rise\' < {0} Angstrom '.format(new_rise) + \
            'or \'percent of reconstruction size\' < {0} % parameters. '.format(new_percentage)
            
            error_message += 'In addition, you can also decrease \'Minimum number of images for 3D reconstruction\'.'
                   
            raise ValueError(error_message)
        
        return symmetry_views_count


    def get_symmetry_transformations_from_helix_input(self, polar_helix, rotational_symmetry_start):
        point_symmetry = self.determine_point_group_symmetry_from_input(polar_helix, rotational_symmetry_start)
        sym_transformations = self.get_symmetry_transformations(point_symmetry)
        
        return sym_transformations
    

    def check_parameter_integritry_and_start_setup(self):
        if EMUtil.get_image_count(self.infile) < self.classno:
            errormsg = 'Requested class number higher than number of images on class average stack.'
            raise ValueError(errormsg)
        
        entered_symmetry_grid = \
        self.generate_rise_rotation_or_pitch_unitnumber_pairs_for_symmetry_grid(self.helical_rise_or_pitch_range,
        self.helical_rise_or_pitch_increment, self.helical_rotation_or_unit_range,
        self.helical_rotation_or_unit_increment)
        
        if self.rise_rot_or_pitch_unit_choice in ['pitch/unit_number']:
            symmetry_grid = SegClassReconstruct().convert_pitch_unit_grid_to_rise_rotation_grid(entered_symmetry_grid)
        else:
            symmetry_grid = entered_symmetry_grid
        
        ori_helixwidthpix = int(round(self.helixwidth / self.ori_pixelsize))
        segment_size_in_pixel, trimmed_image_size, helixheight_pix = self.determine_trimmed_image_size(self.infile,
        self.percent_reconstruction_size, ori_helixwidthpix, self.ori_pixelsize)
        
        symmetry_seq = symmetry_grid.ravel()
        entered_symmetry_seq = entered_symmetry_grid.ravel()
        
        if not self.local_grid_option:
            sym_transformations = self.get_symmetry_transformations_from_helix_input(self.polar_helix,
            self.rotational_symmetry_start)
            
            minimum_image_count_3d = self.minimum_image_count_3d / float(len(sym_transformations))
            for each_symmetry_pair in symmetry_seq:
                views = self.determine_symmetry_view_count(segment_size_in_pixel, trimmed_image_size,
                each_symmetry_pair, self.ori_pixelsize, minimum_image_count_3d)
        else:
            minimum_image_count_3d = self.minimum_image_count_3d
                
        return symmetry_seq, entered_symmetry_seq, minimum_image_count_3d
 

class SegClassReconstruct(SegClassReconstruct3d): 

    def make_named_tuple_for_reconstruction(self):
        rec_named_tuple = namedtuple('rec_parameters', 'stack_id local_id phi theta psi x_shift y_shift ' + \
        'inplane_angle mirror seg_ref_id')
        
        return rec_named_tuple
    
    
    def get_alignment_parameters_from_infile(self, infile):
        segment = EMData()
        segment.read_image(infile, 0)
        segment_transform = segment.get_attr('xform.projection')
        determined = segment_transform.get_params('spider')
        rec_parameters = self.make_named_tuple_for_reconstruction()
        
        alignment_parameters = [rec_parameters(0, 0, determined['phi'], determined['theta'], determined['psi'],
        -determined['tx'], -determined['ty'], 0, 1, 0)]
        
        return alignment_parameters, segment
    

    def log_and_stack_alignment_parameters_into_numpy_array(self, alignment_parameters, symmetry_parameters,
    symmetry_rec_loginfo):
        if symmetry_parameters != []:
            self.log.tlog('Asymmetric units count: {0} from segments '.format(len(alignment_parameters)) + \
            '{0}-{1} were prepared for 3D '.format(int(alignment_parameters[0][0]), (alignment_parameters[-1][0])) + \
            'reconstruction with the following alignment parameters:\n{0}'.format(symmetry_rec_loginfo))
            symmetry_parameters = np.vstack(symmetry_parameters)
            
        return symmetry_parameters
    

    def determine_point_group_symmetry_from_input(self, polar_helix, rotational_symmetry):
        """
        >>> from spring.segment3d.segclassreconstruct import SegClassReconstruct
        >>> SegClassReconstruct().determine_point_group_symmetry_from_input('polar', 2)
        'c2'
        >>> SegClassReconstruct().determine_point_group_symmetry_from_input('apolar', 4)
        'd4'
        """
        if polar_helix in ['apolar']:
            dihedral_symmetry = 'd'
        else:
            dihedral_symmetry = 'c'
        
        point_symmetry = dihedral_symmetry + str(rotational_symmetry)
            
        return point_symmetry
    

    def reconstruct_volume_from_class_average(self, infile, vol_name, segment_size_in_pixel, trimmed_image_size,
    symmetry_views_count, each_rec_number, each_symmetry_pair, point_symmetry, montage_reprj, montage_power):
        if symmetry_views_count is None:
            symmetry_views_count = self.determine_symmetry_view_count(segment_size_in_pixel, trimmed_image_size,
            each_symmetry_pair, self.pixelsize, self.minimum_image_count_3d)
            
        alignment_parameters, class_avg = self.get_alignment_parameters_from_infile(infile)
        
        self.log.fcttolog()
        self.log.in_progress_log()
        
        gc.collect()

        rec_volume, fftvol, weight = self.setup_reconstructor(trimmed_image_size)
        
        rec_stack_info = self.make_rec_stack_info()
        symmetry_helix_volume = self.get_symmetry_file_name(each_symmetry_pair, vol_name)
            
        rec_stack = rec_stack_info(os.path.join(self.tempdir, 'rec_stack.hdf'), symmetry_helix_volume,
        trimmed_image_size)

        rec_volume, symmetry_parameters, symmetry_rec_loginfo, Euler_angles_rec = \
        self.compute_and_write_symmetry_related_views(rec_volume, infile, alignment_parameters, each_symmetry_pair,
        self.pixelsize, symmetry_views_count, trimmed_image_size, rec_stack, point_symmetry)
        
        symmetry_parameters = self.log_and_stack_alignment_parameters_into_numpy_array(alignment_parameters,
        symmetry_parameters, symmetry_rec_loginfo)
        
        symmetry_phi_angles = np.array(symmetry_parameters)[:,1]
        diff_angles = np.diff(np.sort(symmetry_phi_angles))
        
        vol = rec_volume.finish(True)
        
        symmetry_helix_volume, variance = \
        self.write_reconstruction_file_and_set_isotropic_pixelsize(each_symmetry_pair, symmetry_helix_volume,
        self.pixelsize, fftvol, point_symmetry)
            
        helical_error, central_prj, central_class_img = \
        self.determine_mean_ccc_deviation_within_symmetry_related_views(rec_stack, Euler_angles_rec, self.helixwidthpix,
        True, True, True)

        os.remove(rec_stack.file_name)
        if not self.keep_intermediate_files:
            os.remove(symmetry_helix_volume)
            symmetry_helix_volume = None
        else:
            self.write_out_side_by_side_display_of_images_and_power_spectra(central_class_img[0], central_prj[0], 
            each_rec_number, montage_reprj, montage_power, each_symmetry_pair)
        
        avg_azimuth_sampling = np.mean(diff_angles)
        dev_azimuth_sampling = np.std(diff_angles)
        
        sym_vol_nt = self.make_symmetry_vol_named_tuple()
        vol_stat = sym_vol_nt(*((symmetry_helix_volume, variance, avg_azimuth_sampling, dev_azimuth_sampling) + \
                                helical_error))
            
        return vol_stat


    def get_symmetry_file_name(self, each_symmetry_pair, vol_name):
        if self.rise_rot_or_pitch_unit_choice == 'pitch/unit_number':
            pitch_unit_pair = self.convert_rise_rotation_pair_to_pitch_unit_pair(each_symmetry_pair)
            
            symmetry_helix_volume = self.generate_symmetry_grid_file_name(pitch_unit_pair, vol_name,
            self.rise_rot_or_pitch_unit_choice)
        else:
            symmetry_helix_volume = self.generate_symmetry_grid_file_name(each_symmetry_pair, vol_name)
            
        return symmetry_helix_volume


    def set_isotropic_pixelsize_in_volume(self, pixelsize, volume):
        volume.set_attr('apix_x', pixelsize)
        volume.set_attr('apix_y', pixelsize)
        volume.set_attr('apix_z', pixelsize)
        
        return volume
    

    def write_reconstruction_file_and_set_isotropic_pixelsize(self, each_symmetry_pair, symmetry_helix_volume,
    pixelsize, rec_vol, point_symmetry):
        stat = Micrograph().get_statistics_from_image(rec_vol)

        rec_vol = self.set_isotropic_pixelsize_in_volume(pixelsize, rec_vol)
        rec_vol = self.set_header_with_helical_parameters(each_symmetry_pair, rec_vol, point_symmetry)
        rec_vol.write_image(symmetry_helix_volume)

        return symmetry_helix_volume, stat.sigma
    

    def generate_isosurface_or_slice_views_from_volume_if_demanded(self, isosurface_stack, slice_stack, each_rec_number,
    each_symmetry_pair, helix_volume, noise_data=False):
        if not noise_data:
            try:
                trimmed_helix_volume = Util.window(helix_volume, int(1.1*self.helixwidthpix),
                int(1.1*self.helixwidthpix), helix_volume.get_zsize(), 0, 0, 0)
            except:
                trimmed_helix_volume = helix_volume
            if self.isosurface_projection_option:
                if sys.argv[0].endswith('mpi'):
                    isosurface_stack = self.tempdir + isosurface_stack
                isosurface_view = self.generate_isosurface_view_from_reconstruction(trimmed_helix_volume,
                each_symmetry_pair, self.pixelsize, each_rec_number)
                
                isosurface_view.write_image(isosurface_stack, each_rec_number)
            if self.slice_views_option:
                if sys.argv[0].endswith('mpi'):
                    slice_stack = self.tempdir + slice_stack
                slice_view = self.generate_slice_view_from_reconstruction(trimmed_helix_volume, each_symmetry_pair,
                self.pixelsize, each_rec_number)
                
                slice_view.write_image(slice_stack, each_rec_number)
        
        return isosurface_stack, slice_stack
                
                
    def prepare_segclassreconstruct(self):
        self.log.fcttolog()
        
        self.infile = self.clear_previous_alignment_parameters_and_copy_class_to_local_directory(self.infile,
        self.classno, self.out_of_plane_tilt_angle)
        
        if self.centeroption:
            self.centered_class_avg = '{0}_centered{1}'.format(os.path.splitext(self.infile)[0],
            os.path.splitext(self.infile)[-1])
            
            self.infile = self.center_class_avg(self.infile, self.centered_class_avg, self.helixwidthpix,
            self.percent_reconstruction_size, self.out_of_plane_tilt_angle)

        return self.infile


    def make_symmetry_vol_named_tuple(self):
        sym_vol_nt = namedtuple('symmetry_vol', 'helix_vol variance avg_azimuth_sampling dev_azimuth_sampling ' + \
        self.get_helical_error_string())
        return sym_vol_nt


    def reconstruct_volumes_for_each_symmetry_pair(self, symmetry_sequence, total_symmetry_pair_id, infile, vol_name,
    montage_repr, montage_power):
        self.log.fcttolog()
        
        segment_size_in_pixel, trimmed_image_size, helixheight_pix = self.determine_trimmed_image_size(self.infile,
        self.percent_reconstruction_size, self.helixwidthpix, self.pixelsize)
        
        if self.local_grid_option:
            if not sys.argv[0].endswith('mpi'):
                symmetry_views_count = self.determine_symmetry_view_count_for_average_helical_rise(symmetry_sequence,
                segment_size_in_pixel, helixheight_pix)
            if sys.argv[0].endswith('mpi'):
                symmetry_views_count = \
                self.determine_symmetry_view_count_for_average_helical_rise(self.total_symmetry_sequence,
                segment_size_in_pixel, helixheight_pix)
        else:
            symmetry_views_count = None
            
        point_symmetry = self.determine_point_group_symmetry_from_input(self.polar_helix,
        self.rotational_symmetry_start)
        
        symmetry_helix_volumes = []
        for each_rec_number, each_symmetry_pair in enumerate(symmetry_sequence):
            
            vol_stat = self.reconstruct_volume_from_class_average(infile, vol_name, segment_size_in_pixel,
            trimmed_image_size, symmetry_views_count, each_rec_number, each_symmetry_pair, point_symmetry, montage_repr,
            montage_power)
            
            self.log.tlog('Reconstruction: {0}, '.format(total_symmetry_pair_id[each_rec_number]) + \
            'symmetry pair: {0} completed: {1}'.format(vol_stat.helix_vol, each_symmetry_pair.__str__()))
            symmetry_helix_volumes.append(vol_stat)
            
        return symmetry_helix_volumes
    
    
    def get_correct_primary_and_secondary_variables_for_database(self, rise_rot_or_pitch_unit_choice,
    ssecondary_variable=''):
        """
        >>> from spring.segment3d.segclassreconstruct import SegClassReconstruct
        >>> s = SegClassReconstruct()
        >>> hel_rot_n_u_t = 'Helical rotation or number of units per turn'
        >>> s.get_correct_primary_and_secondary_variables_for_database('rise/rotation', hel_rot_n_u_t)
        ('Helical rise in Angstrom', 'Helical rotation in degrees')
        >>> s.get_correct_primary_and_secondary_variables_for_database('pitch/number_of_unit_per_turn')
        ('Helical pitch in Angstrom', 'Number of units per turn')
        >>> s.get_correct_primary_and_secondary_variables_for_database('rise/rotation', None)
        ('Helical rise in Angstrom', '')
        """
        if rise_rot_or_pitch_unit_choice == 'rise/rotation':
            primary_variable = 'Helical rise in Angstrom'
            secondary_variable = 'Helical rotation in degrees'
        else:
            primary_variable = 'Helical pitch in Angstrom'
            secondary_variable = 'Number of units per turn'
        
        if ssecondary_variable is None:
            secondary_variable = ''
            
        return primary_variable, secondary_variable
    

    def get_correct_primary_and_secondary_variables_from_database(self, primary_variable, secondary_variable):
        """
        >>> from spring.segment3d.segclassreconstruct import SegClassReconstruct
        >>> s = SegClassReconstruct()
        >>> s.get_correct_primary_and_secondary_variables_from_database('Helical rise in Angstrom', 'Helical rotation in degrees')
        ('rise/rotation', 'helical_rise_or_pitch', 'helical_rotation_or_number_of_units_per_turn')
        >>> s.get_correct_primary_and_secondary_variables_from_database('Helical pitch in Angstrom', 'Number of units per turn')
        ('pitch/unit_number', 'helical_rise_or_pitch', 'helical_rotation_or_number_of_units_per_turn')
        """
        if primary_variable == 'Helical rise in Angstrom' and secondary_variable == 'Helical rotation in degrees':
            rise_rot_or_pitch_unit_choice = 'rise/rotation'
        elif primary_variable == 'Helical pitch in Angstrom' and secondary_variable == 'Number of units per turn':
            rise_rot_or_pitch_unit_choice = 'pitch/unit_number'
            
        primary_variable = 'helical_rise_or_pitch'
        secondary_variable = 'helical_rotation_or_number_of_units_per_turn'
        
        return rise_rot_or_pitch_unit_choice, primary_variable, secondary_variable
            
    
    def enter_starting_parameters_of_grid_search(self, primary_variable, secondary_variable, primary_range, primary_inc,
    secondary_range, second_inc):
        primary_min, primary_max = primary_range
        grid_cycle = GridTable()
        grid_cycle.dirname = os.path.abspath(os.curdir)
        grid_cycle.primary_variable = primary_variable
        grid_cycle.primary_min = primary_min
        grid_cycle.primary_max = primary_max
        grid_cycle.primary_inc = primary_inc
        
        second_min, second_max = secondary_range
        grid_cycle.secondary_variable = secondary_variable
        grid_cycle.second_min = second_min
        grid_cycle.second_max = second_max
        grid_cycle.second_inc = second_inc
        
        return grid_cycle
    

    def enter_grid_values_in_database(self, symmetry_sequence, symmetry_helix_volumes, montaged_reprojection_stack,
    montaged_power_stack):
        self.log.fcttolog()
        grid_session = SpringDataBase().setup_sqlite_db(grid_base, 'grid.db')
        
        primary_variable, secondary_variable = \
        self.get_correct_primary_and_secondary_variables_for_database(self.rise_rot_or_pitch_unit_choice)
            
        grid_cycle = self.enter_starting_parameters_of_grid_search(primary_variable, secondary_variable,
        self.helical_rise_or_pitch_range, self.helical_rise_or_pitch_increment, self.helical_rotation_or_unit_range,
        self.helical_rotation_or_unit_increment)
        
        grid_session.add(grid_cycle)
            
        for each_grid_id, (each_rise_or_pitch, each_rotation_or_unitnumber) in enumerate(symmetry_sequence):
            grid_refine = GridRefineTable()
            grid_refine.grid = grid_cycle

            if self.keep_intermediate_files:
                vol_name = os.path.abspath(symmetry_helix_volumes[each_grid_id].helix_vol)
                mont_reproj = (os.path.abspath(montaged_reprojection_stack), each_grid_id)
                mont_power = (os.path.abspath(montaged_power_stack), each_grid_id)
                em_files_3d = [vol_name]
                em_files_2d = [mont_reproj, mont_power]
            else:
                em_files_3d = None
                em_files_2d = None

            grid_refine.em_files_2d = em_files_2d
            grid_refine.em_files_3d = em_files_3d
            grid_refine.primary_value = each_rise_or_pitch
            grid_refine.secondary_value = each_rotation_or_unitnumber

            for each_field in symmetry_helix_volumes[each_grid_id]._fields:
                setattr(grid_refine, each_field, getattr(symmetry_helix_volumes[each_grid_id], each_field))
        
            grid_session.add(grid_refine)
            
        grid_session.commit()
        
        return grid_session
    

    def cleanup_at_end(self):
        os.rmdir(self.tempdir)
        

    def perform_reconstructions_from_class_for_symmetry_combinations(self):
        
        self.symmetry_sequence, entered_symmetry_seq, self.minimum_image_count_3d = \
        self.check_parameter_integritry_and_start_setup()
        
        OpenMpi().setup_and_start_mpi_version_if_demanded(self.mpi_option, self.feature_set, self.cpu_count)
        self.tempdir = Temporary().mktmpdir(self.temppath)
        
        self.infile = self.prepare_segclassreconstruct()
        self.log.plog(10)
        
        total_symmetry_pair_id = list(range(len(self.symmetry_sequence)))
        self.log.plog(40)
            
        symmetry_helix_volumes = self.reconstruct_volumes_for_each_symmetry_pair(self.symmetry_sequence,
        total_symmetry_pair_id, self.infile, self.volume_name, self.montaged_reprojection_stack,
        self.montaged_power_stack)
        
        self.log.plog(80)

        self.enter_grid_values_in_database(entered_symmetry_seq, symmetry_helix_volumes,
        self.montaged_reprojection_stack, self.montaged_power_stack)

        self.log.plog(90)
            
        self.cleanup_at_end()
        self.log.endlog(self.feature_set)
        
def main():
    # Option handling
    parset = SegClassReconstructPar()
    mergeparset = OptHandler(parset)

    ######## Program
    class_average = SegClassReconstruct(mergeparset)
    class_average.perform_reconstructions_from_class_for_symmetry_combinations()


if __name__ == '__main__':
    main()



# Author: Carsten Sachse 08-Jun-2011
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details
"""
Program to compute 3D reconstruction from a single class average using a range of different helical symmetries
"""
from EMAN2 import EMData, EMNumPy, Transform, Vec2f, Util
import os
from projection import prgs
from spring.csinfrastr.csfeatures import Features
from spring.csinfrastr.cslogger import Logger
from spring.particle2d.particlealign2d import ParticleAlign2d
from utilities import set_params_proj, image_decimate

import numpy as np


class SegClassReconstructPar(object):
    """
    Class to initiate default dictionary with input parameters including help and range values and status dictionary
    """
    def __init__(self):
        # package/program identity
        self.package = 'emspring'
        self.progname = 'segclassreconstruct'
        self.proginfo = __doc__

        self.code_files = ['segclassreconstruct_prep', 'segclassreconstruct_assist', self.progname, 
        self.progname + '_mpi']

        self.segclassreconstruct_features = Features()
        self.feature_set = self.segclassreconstruct_features.setup(self)
        
        self.define_parameters_and_their_properties()
        self.define_program_states()


    def define_parameters_and_their_properties(self):
        self.feature_set = self.segclassreconstruct_features.set_class_avg_stack(self.feature_set)
        self.feature_set = self.set_volume_prefix_symmetry_grid(self.feature_set)
        
        self.feature_set = self.segclassreconstruct_features.set_class_number_to_be_analyzed(self.feature_set)
        
        self.feature_set = \
        self.segclassreconstruct_features.set_helical_symmetry_reconstruction_series(self.feature_set)
        
        self.feature_set = self.segclassreconstruct_features.set_keep_intermediate_files_option(self.feature_set, 
        'Keep 3D reconstruction files. Caution: depending on the size of the helical symmetry grid this can ' + \
        'generate vast amounts of data.')
        
        self.feature_set = self.set_montage_class_reprj_out_stack(self.feature_set)
        self.feature_set = self.set_montage_class_reprj_periodogram_out_stack(self.feature_set)

        self.feature_set = self.segclassreconstruct_features.set_binning_option(self.feature_set, default=True)
        self.feature_set = self.segclassreconstruct_features.set_binning_factor(self.feature_set, binfactor=3, image='segments')

        self.feature_set = self.segclassreconstruct_features.set_rotational_symmetry(self.feature_set)
        self.feature_set = self.segclassreconstruct_features.set_polar_apolar_helix_choice(self.feature_set)
        self.feature_set = self.segclassreconstruct_features.set_out_of_plane_tilt_angle(self.feature_set)
        
        self.feature_set = self.segclassreconstruct_features.set_volume_size_to_be_reconstructed(self.feature_set)
        self.feature_set = self.set_minimum_image_count_reconstruction(self.feature_set)
        self.feature_set = self.set_local_grid_option(self.feature_set)
        self.feature_set = self.set_center_and_rotation_option(self.feature_set)
        self.feature_set = self.segclassreconstruct_features.set_mpi(self.feature_set)
        self.feature_set = self.segclassreconstruct_features.set_ncpus(self.feature_set)
        self.feature_set = self.segclassreconstruct_features.set_temppath(self.feature_set)


    def define_program_states(self):
        self.feature_set.program_states['prepare_segclassreconstruct']='Prepare 3D reconstruction'
        self.feature_set.program_states['reconstruct_volumes_for_each_symmetry_pair']='Reconstruct_helically ' + \
        'symmetric volumes from class average'
        self.feature_set.program_states['enter_grid_values_in_database']='Enter grid values in database'


    def set_center_and_rotation_option(self, feature_set):
        inp7 = 'Center option'
        feature_set.parameters[inp7] = bool(True)
        feature_set.hints[inp7] = 'Segments are centered and rotationally aligned with respect to their helix axis'
        feature_set.level[inp7]='intermediate'

        return feature_set


#     def set_symgrid_keep_option(self, feature_set):
#         inp15 = 'Keep symmetry grid reconstructions'
#         feature_set.parameters[inp15]=bool(False)
#         feature_set.hints[inp15]='Keep reconstructions that are generated for each symmetry pair - volumes are ' + \
#         'deleted otherwise'
#         
#         return feature_set
    
    
    def set_volume_prefix_symmetry_grid(self, feature_set):
        inp9 = 'Volume name'
        feature_set.parameters[inp9]='recvol.hdf'
        feature_set.properties[inp9]=feature_set.file_properties(1,['hdf'],'saveFile')
        feature_set.hints[inp9]='Output name for volumes of grid search (completion to ' + \
        '\'name_riseXXX_rot_XXX.ext\' or \'name_pitchXXX_unitno_XXX.ext\): accepted image file formats ({0})'.\
        format(', '.join(feature_set.properties[inp9].ext))
        
        feature_set.level[inp9]='beginner'
        
        return feature_set
    
    
    def set_minimum_image_count_reconstruction(self, feature_set):
        inp5 = 'Minimum number of images for 3D reconstruction'
        feature_set.parameters[inp5] = int(8)
        feature_set.hints[inp5] = 'Minimum number of images required for 3D reconstruction. Below this value no ' + \
        '3D reconstruction possible.'
        feature_set.properties[inp5] = feature_set.Range(4, 100, 1)
        feature_set.level[inp5]='expert'

        return feature_set
    
    
    def set_local_grid_option(self, feature_set):
        inp15 = 'Local symmetry grid search'
        feature_set.parameters[inp15]=bool(False)
        feature_set.hints[inp15]='Local symmetry grid search fixes the number of images that are inserted in ' + \
        'each reconstruction'
        feature_set.level[inp15]='expert'
        
        return feature_set
    
    
    def set_montage_class_reprj_out_stack(self, feature_set):
        inp2 = 'Montage stack of class average vs. reprojection images'
        feature_set.parameters[inp2] = 'reprojection_montage.hdf'
        feature_set.properties[inp2] = feature_set.file_properties(1, ['hdf'], 'saveFile')
        feature_set.hints[inp2] = 'Output montage of images from class average vs reprojection stack of 3D ' + \
        'reconstructions: accepted image file formats ({0})'.format(', '.join(feature_set.properties[inp2].ext))
        feature_set.relatives[inp2]='Keep intermediate files'
        feature_set.level[inp2]='intermediate'

        return feature_set
    
    
    def set_montage_class_reprj_periodogram_out_stack(self, feature_set):
        inp2 = 'Montage stack of class average vs. reprojection power spectra'
        feature_set.parameters[inp2] = 'power_montage.hdf'
        feature_set.properties[inp2] = feature_set.file_properties(1, ['hdf'], 'saveFile')
        
        feature_set.hints[inp2] = 'Output montage of power spectra from class average and reprojection stack of ' + \
        '3D reconstructions: accepted image file formats (%s)' % (', '.join(feature_set.properties[inp2].ext))
        
        feature_set.relatives[inp2]='Keep intermediate files'
        feature_set.level[inp2]='intermediate'
        
        return feature_set
    
    
#     def set_generate_surface_slice_views_option(self, feature_set):
#         inp15 = 'Volume slice views'
#         feature_set.parameters[inp15]=bool(False)
#         feature_set.hints[inp15]='Produce stack of volume slice views'
#         
#         return feature_set
#     
# 
#     def set_slice_view_stack(self, feature_set):
#         inp9 = 'Volume slice view stack'
#         feature_set.parameters[inp9]='volume_slice_views.hdf'
#         feature_set.properties[inp9]=feature_set.file_properties(1,['hdf'],'saveFile')
#         feature_set.hints[inp9]='Volume slice view of each symmetry pair reconstruction for visual inspection'
#         feature_set.relatives[inp9]='Volume slice views'
#         
#         return feature_set
#     
#     
#     def set_generate_noise_reprojections_option(self, feature_set):
#         inp15 = 'Noise volumes for normalization'
#         feature_set.parameters[inp15]=bool(True)
#         feature_set.hints[inp15]='Generate pure noise volumes for each symmetry pair that are used to normalize ' + \
#         'similarity comparison'
#         
#         feature_set.level[inp15]='expert'
#         
#         return feature_set
#     
#     
#     def set_noise_image(self, feature_set):
#         inp9 = 'Noise image'
#         feature_set.parameters[inp9]='noise_image.hdf'
#         feature_set.properties[inp9]=feature_set.file_properties(1,['hdf'],'saveFile')
#         feature_set.hints[inp9]='Pure noise image that is used for reconstruction for each symmetry pair and can ' + \
#         'be used later in Seggridcompare to normalize similarity measures'
#         feature_set.relatives[inp9]='Noise volumes for normalization'
#         feature_set.level[inp9]='expert'
#         
#         return feature_set
    
    
class SegClassReconstructPreparation(object):
    """
    * Class that holds functions to insert_image_slices_in_3D_volume helically symmetric volumes
    """
    def __init__(self, parset = None):
        self.log = Logger()
        if parset is not None:
            self.feature_set = parset
            p = self.feature_set.parameters

            self.infile = p['Class average stack']
            self.volume_name = p['Volume name']
            self.outfile = self.volume_name
            self.classno = p['Class number to be analyzed']
            self.keep_intermediate_files = p['Keep intermediate files']
            self.montaged_reprojection_stack = p['Montage stack of class average vs. reprojection images']
            self.montaged_power_stack = p['Montage stack of class average vs. reprojection power spectra']
            
            self.ori_pixelsize = p['Pixel size in Angstrom']
            self.binoption=p['Binning option']
            self.binfactor=p['Binning factor']
            if not self.binoption:
                self.binfactor = 1
            elif self.binfactor == 1 and self.binoption:
                self.binoption = False
            self.pixelsize = self.ori_pixelsize * self.binfactor
            
            self.helix_inner_width = p['Estimated helix inner and outer diameter in Angstrom'][0]
            self.helixwidth = p['Estimated helix inner and outer diameter in Angstrom'][1]
            self.helix_inner_widthpix = int(round(self.helix_inner_width / self.pixelsize))
            self.helixwidthpix = int(round(self.helixwidth / self.pixelsize))
            
            self.rise_rot_or_pitch_unit_choice = p['Helical rise/rotation or pitch/number of units per turn choice']
            self.helical_rise_or_pitch_range = p['Range of helical rise or pitch search in Angstrom']
            self.helical_rotation_or_unit_range = p['Range of helical rotation in degrees or number of units per ' + \
            'turn search']
            
            self.helical_rise_or_pitch_increment = p['Increments of helical symmetry steps in Angstrom or degrees'][0]
            self.helical_rotation_or_unit_increment = p['Increments of helical symmetry steps in Angstrom or degrees'][1]
            self.rotational_symmetry_start = p['Rotational symmetry']
            self.polar_helix = p['Helix polarity']
            self.out_of_plane_tilt_angle = p['Out-of-plane tilt angle']
            
            self.minimum_image_count_3d = p['Minimum number of images for 3D reconstruction']
            self.percent_reconstruction_size = p['Percent of image reconstruction size']
            self.centeroption=p['Center option']
            self.local_grid_option = p['Local symmetry grid search']
            
            self.mpi_option = p['MPI option']
            self.cpu_count = p['Number of CPUs']
            self.temppath=p['Temporary directory']
    

    def convert_rise_rotation_or_pitch_unitnumber_series_to_grid_of_tuples(self, rise_series, rotation_series):
        """
        >>> from spring.segment3d.segclassreconstruct import SegClassReconstruct                                                                          
        >>> s = SegClassReconstruct()
        >>> s.convert_rise_rotation_or_pitch_unitnumber_series_to_grid_of_tuples(range(2), range(3))
        array([[(0.0, 0.0), (0.0, 1.0), (0.0, 2.0)],
               [(1.0, 0.0), (1.0, 1.0), (1.0, 2.0)]], dtype=object)
        """
        
        symmetry_grid = np.zeros((len(rise_series), len(rotation_series)), dtype=tuple)
        for each_rise_index, each_rise in enumerate(rise_series):
            for each_rotation_index, each_rotation in enumerate(rotation_series):
                symmetry_grid[each_rise_index][each_rotation_index] = (float(each_rise), float(each_rotation))
        
        return symmetry_grid


    def generate_series(self, helical_rise_range, helical_rise_increment):
        rise_count = int(round((max(helical_rise_range) - min(helical_rise_range)) / helical_rise_increment)) + 1
        rise_series = np.linspace(min(helical_rise_range), max(helical_rise_range), rise_count)

        return rise_series.tolist()


    def generate_unique_rise_rotation_or_pitch_unitnumber_arrays(self, helical_rise_range, helical_rise_increment,
    helical_rotation_range, helical_rotation_increment):
        """
        >>> from spring.segment3d.segclassreconstruct import SegClassReconstruct                                                                          
        >>> s = SegClassReconstruct()
        >>> s.generate_unique_rise_rotation_or_pitch_unitnumber_arrays([2,4], 2, [10,30], 10)
        ([2.0, 4.0], [10.0, 20.0, 30.0])
        >>> s.generate_unique_rise_rotation_or_pitch_unitnumber_arrays((275.0, 305.0), 5.0, (3.8, 4.2), 0.1)
        ([275.0, 280.0, 285.0, 290.0, 295.0, 300.0, 305.0], [3.8, 3.9, 4.0, 4.1, 4.2])
        >>> s.generate_unique_rise_rotation_or_pitch_unitnumber_arrays((270.0, 310.0), 5.0, (3.1, 3.7), 0.1) #doctest: +NORMALIZE_WHITESPACE
        ([270.0, 275.0, 280.0, 285.0, 290.0, 295.0, 300.0, 305.0, 310.0], 
        [3.1, 3.2, 3.3000000000000003, 3.4000000000000004, 3.5, 3.6, 3.7])
        """
        if helical_rotation_range[0] != helical_rotation_range[-1] and helical_rotation_increment == 0:
            helical_rotation_range = (helical_rotation_range[0], helical_rotation_range[0])
        if helical_rotation_increment == 0 or helical_rotation_increment is None:
            helical_rotation_increment = 1
        if helical_rise_range[0] != helical_rise_range[-1] and helical_rise_increment == 0:
            helical_rise_range = (helical_rise_range[0], helical_rise_range[0])
        if helical_rise_increment == 0 or helical_rise_increment is None:
            helical_rise_increment = 1
            
        rise_series = self.generate_series(helical_rise_range, helical_rise_increment)
        rotation_series = self.generate_series(helical_rotation_range, helical_rotation_increment)
        
        return rise_series, rotation_series
    

    def generate_rise_rotation_or_pitch_unitnumber_pairs_for_symmetry_grid(self, helical_rise_range, helical_rise_increment,
                                                       helical_rotation_range, helical_rotation_increment):
        """
        * Function to generate pairs of rise and rotation for symmetry grid search
        
        #. Usage: symmetry_grid = generate_rise_rotation_or_pitch_unitnumber_pairs_for_symmetry_grid(helical_rise_range,
        helical_rise_increment, helical_rotation_range, helical_rotation_increment)
                                                                                
        #. Input: helical_rise_range, helical_rotation_range = list of start and end values
        helical_rise_increment, helical_rotation_increment = number of increment
        #. Output: list of symmetry pairs of rise and rotation
        
        >>> from spring.segment3d.segclassreconstruct import SegClassReconstruct                                                                          
        >>> s = SegClassReconstruct()
        >>> s.generate_rise_rotation_or_pitch_unitnumber_pairs_for_symmetry_grid([2,4], 2, [10,30], 10)
        array([[(2.0, 10.0), (2.0, 20.0), (2.0, 30.0)],
               [(4.0, 10.0), (4.0, 20.0), (4.0, 30.0)]], dtype=object)
               
        >>> s.generate_rise_rotation_or_pitch_unitnumber_pairs_for_symmetry_grid([2,3], 0, [10,20], 2)
        array([[(2.0, 10.0), (2.0, 12.0), (2.0, 14.0), (2.0, 16.0), (2.0, 18.0),
                (2.0, 20.0)]], dtype=object)
                
        >>> s.generate_rise_rotation_or_pitch_unitnumber_pairs_for_symmetry_grid([2,3], 1, [10,10], 0)
        array([[(2.0, 10.0)],
               [(3.0, 10.0)]], dtype=object)
               
        >>> s.generate_rise_rotation_or_pitch_unitnumber_pairs_for_symmetry_grid((275.0, 305.0), 5.0, (3.8, 4.2), 0.1)
        array([[(275.0, 3.8), (275.0, 3.9), (275.0, 4.0), (275.0, 4.1),
                (275.0, 4.2)],
               [(280.0, 3.8), (280.0, 3.9), (280.0, 4.0), (280.0, 4.1),
                (280.0, 4.2)],
               [(285.0, 3.8), (285.0, 3.9), (285.0, 4.0), (285.0, 4.1),
                (285.0, 4.2)],
               [(290.0, 3.8), (290.0, 3.9), (290.0, 4.0), (290.0, 4.1),
                (290.0, 4.2)],
               [(295.0, 3.8), (295.0, 3.9), (295.0, 4.0), (295.0, 4.1),
                (295.0, 4.2)],
               [(300.0, 3.8), (300.0, 3.9), (300.0, 4.0), (300.0, 4.1),
                (300.0, 4.2)],
               [(305.0, 3.8), (305.0, 3.9), (305.0, 4.0), (305.0, 4.1),
                (305.0, 4.2)]], dtype=object)
        """
                                                       
        rise_series, rotation_series = self.generate_unique_rise_rotation_or_pitch_unitnumber_arrays(helical_rise_range,
        helical_rise_increment, helical_rotation_range, helical_rotation_increment)
        
        symmetry_grid = self.convert_rise_rotation_or_pitch_unitnumber_series_to_grid_of_tuples(rise_series,
        rotation_series)
                
        return symmetry_grid
    
    
    def convert_rise_rotation_pair_to_pitch_unit_pair(self, each_rise_rotation_pair):
        """
        >>> from spring.segment3d.segclassreconstruct import SegClassReconstruct
        >>> s = SegClassReconstruct()
        >>> s.convert_rise_rotation_pair_to_pitch_unit_pair((1.408, 22.03))
        (23.00862460281434, 16.34135270086246)
        >>> s.convert_rise_rotation_pair_to_pitch_unit_pair((1.408, -22.03))
        (-23.00862460281434, -16.34135270086246)
        >>> s.convert_rise_rotation_pair_to_pitch_unit_pair((-1.408, -22.03))
        (23.00862460281434, -16.34135270086246)
        >>> s.convert_rise_rotation_pair_to_pitch_unit_pair((-1.408, 22.03))
        (-23.00862460281434, 16.34135270086246)
        """
        rise = each_rise_rotation_pair[0]
        rotation = each_rise_rotation_pair[1]
        unit_number = 360 / rotation
        pitch = rise * unit_number
        pitch_unit_pair = (pitch, unit_number)
        
        return pitch_unit_pair
    
    
    def convert_pitch_unit_pair_to_rise_rotation_pairs(self, pitch, unit_number):
        """
        >>> from spring.segment3d.segclassreconstruct import SegClassReconstruct
        >>> s = SegClassReconstruct()
        >>> s.convert_pitch_unit_pair_to_rise_rotation_pairs(23, 16.3333334)
        (1.4081632595585172, 22.040816236568094)
        >>> s.convert_pitch_unit_pair_to_rise_rotation_pairs(-23, -16.3333334)
        (1.4081632595585172, -22.040816236568094)
        >>> s.convert_pitch_unit_pair_to_rise_rotation_pairs(23, -16.3333334)
        (-1.4081632595585172, -22.040816236568094)
        >>> s.convert_pitch_unit_pair_to_rise_rotation_pairs(-23, 16.3333334)
        (-1.4081632595585172, 22.040816236568094)
        """
        rise = pitch / unit_number
        rotation = 360.0 / unit_number
        rise_rotation_pair = (rise, rotation)
        
        return rise_rotation_pair
    
    
    def convert_rise_rotation_grid_to_pitch_unit_grid(self, rise_rot_grid, shape=None):
        """
        >>> from spring.segment3d.segclassreconstruct import SegClassReconstruct 
        >>> s = SegClassReconstruct()
        >>> r_r_grid = s.generate_rise_rotation_or_pitch_unitnumber_pairs_for_symmetry_grid([10,30], 10, [10,40], 10) 
        >>> s.convert_rise_rotation_grid_to_pitch_unit_grid(r_r_grid)
        array([[(360.0, 36.0), (180.0, 18.0), (120.0, 12.0), (90.0, 9.0)],
               [(720.0, 36.0), (360.0, 18.0), (240.0, 12.0), (180.0, 9.0)],
               [(1080.0, 36.0), (540.0, 18.0), (360.0, 12.0), (270.0, 9.0)]], dtype=object)
        """
        
        pitch_unit_seq = np.zeros(rise_rot_grid.size, dtype=tuple)
        for each_id, each_rise_rot_pair in enumerate(rise_rot_grid.ravel()):
            pitch_unit_pair = self.convert_rise_rotation_pair_to_pitch_unit_pair(each_rise_rot_pair)
            
            pitch_unit_seq[each_id]=pitch_unit_pair
            
        if shape is None:
            pitch_unit_grid = pitch_unit_seq.reshape(rise_rot_grid.shape)
        else:
            pitch_unit_grid = pitch_unit_seq.reshape(shape)
        
        return pitch_unit_grid


    def convert_pitch_unit_grid_to_rise_rotation_grid(self, pitch_unit_grid, shape=None):
        """
        >>> from spring.segment3d.segclassreconstruct import SegClassReconstruct 
        >>> s = SegClassReconstruct()
        >>> p_u_grid = s.generate_rise_rotation_or_pitch_unitnumber_pairs_for_symmetry_grid([10,30], 10, [10,40], 10) 
        >>> s.convert_pitch_unit_grid_to_rise_rotation_grid(p_u_grid)
        array([[(1.0, 36.0), (0.5, 18.0), (0.3333333333333333, 12.0), (0.25, 9.0)],
               [(2.0, 36.0), (1.0, 18.0), (0.6666666666666666, 12.0), (0.5, 9.0)],
               [(3.0, 36.0), (1.5, 18.0), (1.0, 12.0), (0.75, 9.0)]], dtype=object)
        """
        
        rise_rotation_seq = np.zeros(pitch_unit_grid.size, dtype=tuple)
        for each_index, each_pitch_unit_pair in enumerate(pitch_unit_grid.ravel()):
            rise_rotation_pair = self.convert_pitch_unit_pair_to_rise_rotation_pairs(each_pitch_unit_pair[0],
            each_pitch_unit_pair[1])
            
            rise_rotation_seq[each_index]=rise_rotation_pair
            
        if shape is None:
            rise_rotation_grid = rise_rotation_seq.reshape(pitch_unit_grid.shape)
        else:
            rise_rotation_grid = rise_rotation_seq.reshape(shape)
        
        return rise_rotation_grid
    
    
    def get_standard_proj_parameters(self, out_of_plane_tilt_angle):
        phi = 0.0
        corrected_theta = 90.0 + out_of_plane_tilt_angle
        psi = 270.0
        tx = 0.0
        ty = 0.0
        
        return phi, corrected_theta, psi, tx, ty
    
            
    def set_projection_transform_for_helix_azimuth_zero(self, segment, out_of_plane_tilt_angle):
        phi, theta, psi, sx, sy = self.get_standard_proj_parameters(out_of_plane_tilt_angle)
        
        t2 = Transform({'type':'spider', 'phi':phi, 'theta':theta, 'psi':psi})
        t2.set_trans(Vec2f(-sx, -sy))
        segment.set_attr('xform.projection', t2)
        
        return segment
    

    def clear_previous_alignment_parameters_and_copy_class_to_local_directory(self, infile, classno,
    out_of_plane_tilt_angle):
        class_avg = EMData()
        class_avg.read_image(infile, classno)

        if self.binoption:
            class_avg = image_decimate(class_avg, self.binfactor)

        class_avg = self.set_projection_transform_for_helix_azimuth_zero(class_avg, out_of_plane_tilt_angle)
        local_input_file = os.path.basename(infile)
        class_avg.write_image(local_input_file)
        
        return local_input_file
    
    
class SegClassReconstructCylinderMask(SegClassReconstructPreparation):

    def make_smooth_cylinder_mask(self, outer_diameter_in_pixel, inner_diameter_in_pixel, segment_size_in_pixel,
    segment_height_in_pixel=None, width_falloff=0.1):
        """
        >>> from spring.segment3d.segclassreconstruct import SegClassReconstruct      
        >>> from EMAN2 import EMNumPy
        >>> cylinder = SegClassReconstruct().make_smooth_cylinder_mask(8,4,10)
        >>> EMNumPy.em2numpy(cylinder)[5]
        array([[  6.85500383e-01,   6.85500383e-01,   6.85500383e-01,
                  6.85500383e-01,   6.85500383e-01,   6.85500383e-01,
                  6.85500383e-01,   6.85500383e-01,   6.85500383e-01,
                  6.85500383e-01],
               [  6.85500383e-01,   6.85500383e-01,   6.85500383e-01,
                  5.27863979e-01,   8.76894474e-01,   1.00000000e+00,
                  8.76894474e-01,   5.27863979e-01,   6.85500383e-01,
                  6.85500383e-01],
               [  6.85500383e-01,   6.85500383e-01,   7.57359564e-01,
                  9.67862368e-01,   9.31746781e-01,   9.18525219e-01,
                  9.31746781e-01,   9.67862368e-01,   7.57359564e-01,
                  6.85500383e-01],
               [  6.85500383e-01,   5.27863979e-01,   9.67862368e-01,
                  8.67420673e-01,   6.90981150e-01,   6.20666146e-01,
                  6.90981150e-01,   8.67420673e-01,   9.67862368e-01,
                  5.27863979e-01],
               [  6.85500383e-01,   8.76894474e-01,   9.31746781e-01,
                  6.90981150e-01,   2.57088304e-01,   5.96046448e-08,
                  2.57088304e-01,   6.90981150e-01,   9.31746781e-01,
                  8.76894474e-01],
               [  6.85500383e-01,   1.00000000e+00,   9.18525219e-01,
                  6.20666146e-01,   5.96046448e-08,   5.96046448e-08,
                  5.96046448e-08,   6.20666146e-01,   9.18525219e-01,
                  1.00000000e+00],
               [  6.85500383e-01,   8.76894474e-01,   9.31746781e-01,
                  6.90981150e-01,   2.57088304e-01,   5.96046448e-08,
                  2.57088304e-01,   6.90981150e-01,   9.31746781e-01,
                  8.76894474e-01],
               [  6.85500383e-01,   5.27863979e-01,   9.67862368e-01,
                  8.67420673e-01,   6.90981150e-01,   6.20666146e-01,
                  6.90981150e-01,   8.67420673e-01,   9.67862368e-01,
                  5.27863979e-01],
               [  6.85500383e-01,   6.85500383e-01,   7.57359564e-01,
                  9.67862368e-01,   9.31746781e-01,   9.18525219e-01,
                  9.31746781e-01,   9.67862368e-01,   7.57359564e-01,
                  6.85500383e-01],
               [  6.85500383e-01,   6.85500383e-01,   6.85500383e-01,
                  5.27863979e-01,   8.76894474e-01,   1.00000000e+00,
                  8.76894474e-01,   5.27863979e-01,   6.85500383e-01,
                  6.85500383e-01]], dtype=float32)
        """ 
        if segment_height_in_pixel is None:
            segment_height_in_pixel = segment_size_in_pixel
            
        vol = np.zeros((segment_height_in_pixel, segment_size_in_pixel, segment_size_in_pixel))
        
        circle_plane = ParticleAlign2d().make_smooth_circular_mask(outer_diameter_in_pixel, inner_diameter_in_pixel,
        segment_size_in_pixel, width_falloff)
        circle_plane_np = np.copy(EMNumPy.em2numpy(circle_plane))
        
        for each_plane in list(range(segment_height_in_pixel)):
            vol[each_plane] = circle_plane_np
            
        cylvol = EMNumPy.numpy2em(np.copy(vol))
        
        return cylvol
    
    
    def prepare_volume_locally(self, vol):
        volsize_x=vol.get_xsize()
        volsize_y=vol.get_ysize()
        volsize_z=vol.get_zsize()
        K = 6
        alpha = 1.75
        
#        if volume_size_too_big_reduce_memory_consumption:
#            npad = 1
#        else:
#            npad = 2
            
        npad = 2
        if volsize_x == volsize_z:
            pad_size = int(volsize_x*npad)
            kbx    = Util.KaiserBessel(alpha, K, volsize_x/2, K/(2.*pad_size), pad_size)
            volft = vol.copy()
            volft.divkbsinh(kbx)
            volft = volft.norm_pad(False, npad)
            volft.do_fft_inplace()
            volft.center_origin_fft()
            volft.fft_shuffle()
            kby = None
            kbz = None
        else:
            padsize_x     = volsize_x*npad
            padsize_y     = volsize_y*npad
            padsize_z     = volsize_z*npad
            # support of the window
            kbx    = Util.KaiserBessel(alpha, K, volsize_x/2, K/(2.*padsize_x), padsize_x)
            kby    = Util.KaiserBessel(alpha, K, volsize_y/2, K/(2.*padsize_y), padsize_y)
            kbz    = Util.KaiserBessel(alpha, K, volsize_z/2, K/(2.*padsize_z), padsize_z)
            volft = vol.copy()
            volft.divkbsinh_rect(kbx,kby,kbz)
            volft = volft.norm_pad(False, npad)
            volft.do_fft_inplace()
            volft.center_origin_fft()
            volft.fft_shuffle()
            
        return  volft, kbx, kby, kbz
            
            
    def project_locally(self, vol, params, stack=None):
        volft, kbx, kby, kbz = self.prepare_volume_locally(vol)
        if stack is None:
            projs = []
        else:
            projs = stack
            
        for each_prj_id, each_param in enumerate(params):
            proj = prgs(volft, kbx, each_param, kby, kbz)
            set_params_proj(proj, [each_param[0], each_param[1], each_param[2], -each_param[3], -each_param[4]])
            proj.set_attr_dict({'active':1, 'ctf_applied':0})
            if stack is None:
                projs.append(proj)
            else:
                proj.write_image(stack, each_prj_id)
        
        return projs


# Author: Carsten Sachse 11-Nov-2011
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details

from spring.csinfrastr.csproductivity import OpenMpi
from spring.micprgs.micctfdetermine_mpi import ScanMpi
from spring.micprgs.micexam import MicrographExam, MicrographExamPar

class MicrographExamMpi(MicrographExam, ScanMpi):

    def exam_scans(self):
        self.startup_scan_mpi_programs()
        
        if self.micrograph_files != []:
            self.examine_scans_computing_total_and_local_powerspectra(self.micrograph_files, self.outfiles)
        
        self.end_scan_mpi_programs()

def main():
    parset = MicrographExamPar()
    reduced_parset = OpenMpi().start_main_mpi(parset)
    
    ####### Program
    micrograph = MicrographExamMpi(reduced_parset)
    micrograph.exam_scans()


if __name__ == '__main__':
    main()



# Author: Carsten Sachse 21-Nov-2012
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details
import os
import sys

def main():
    if len(sys.argv) > 1:
        commands = sys.argv[1:]
        os.environ['PYTHONPATH'] = ':'.join(sys.path) + ':' + os.environ['PYTHONPATH']
        if os.path.abspath(commands[0]) == commands[0]:
            os.execve(commands[0], commands, os.environ)
        else:
            from spring.csinfrastr.csproductivity import Support
            complete_path_command = Support().search_path_like_which(commands[0])
            os.execve(complete_path_command, commands, os.environ)


if __name__ == '__main__':
    main()